/***************************************************************************************/
// File: authenticationBean.java
// Date: 02/07/2001
//
// Author: Amelia M. Chan
//
// This file provides methods to authenticate a given username and password.
//
/**************************************************************************************/

package portal.base;

import java.sql.*;
import java.io.*;
import java.util.*;

import portal.bean.*;

public class UserAuthentication
//public class UserAuthentication

{
  private String username;
  private String password;
  private int operatorID;
  private int operatorGroupID;
  private String myQuery;
  private DbBean myDbBean;
  protected DebugWriter writer;
  private Vector queryResult;


  public UserAuthentication()
  {
	  username = "";
	  password = "";
	  operatorGroupID = 0;
	  myQuery = "";
      myDbBean = new DbBean();
      writer = new DebugWriter();
}


  public void setUsername(String name) {
     this.username = name;
  }

  public void setPassword(String passwd) {
     this.password = passwd;
  }

  public void setOperatorGroupID(int num) {
     this.operatorGroupID= num;
  }

  public String getUsername() {
     return username;
  }

  public String getPassword() {
     return password;
  }

  public int getOperatorID() {
       return operatorID;
  }

  public int getOperatorGroupID() {
     return operatorGroupID;
  }

 /********************************************************************
* Method:   authenticate()
* Purpose:  This method executes the following sql statements:
*
* return:   resultSet includes operator_group_id.
*
* Note:     no Need to call getUsername() and getPassword() prior to this method.
********************************************************************/

  public int authenticate (String name, String passwd)  throws Exception
  {
setUsername(name);
setPassword(passwd);
operatorGroupID = 0;	//reset memory from last login

 myQuery = "";
 myQuery="select id, operator_group_id from operator where username ='"+name+"' and password = '"+passwd+"'";
     myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
		if (myDbBean.getNextRecord()){
            int found = myDbBean.getColumnInt("OPERATOR_GROUP_ID");
            int oprID = myDbBean.getColumnInt("ID");
            operatorID = oprID;
            setOperatorGroupID(found);
	    }
     }
     try{
            myDbBean.takeDown();
     }catch(Exception e){
            System.out.println("Error Connection:    "+e);
     }

     return getOperatorGroupID();
 }




  /********************************************************************
  * Method:  Vector getISPVN_IDList()
  *
  * Purpose:  This method returns a list of name from ispvn by
  *           the given Ispvn.
  *
  * Note:     Needs to call setOperatorGroupID prior to this method.
  ********************************************************************/

    public Vector getISPVN_IDList(int id) throws Exception
    {
    Vector resultV = new Vector();

  	setOperatorGroupID(id);
    myQuery = "";
    myQuery="select ispvn_id from operator_ispvn where operator_group_id = "+ id;

       myDbBean.setMyQuery(myQuery);
  	   myDbBean.execute_sql(myQuery);

       if (myDbBean.loadData()) {
  	      while (myDbBean.getNextRecord()){
			 Integer num = new Integer (myDbBean.getColumnInt("ISPVN_ID"));
			 resultV.addElement(num);
          }
       }
       try{
            myDbBean.takeDown();
       }catch(Exception e){
            System.out.println("Error Connection:    "+e);
       }

       return resultV;
    }




      /********************************************************************
	  * Method:  Vector getIspvnName()
	  *
	  * Purpose:  This method returns a list of name from ispvn by
	  *           the given Ispvn.
	  * return:  resultSet includes one or more ispvn_id.
	  *
	  ********************************************************************/

public Vector getIspvnName(Vector vIspvn) throws Exception
{

  Vector resultV = new Vector();
  for (int i=0; i < vIspvn.size(); i++ )
   {
  	 myQuery = "";
  	 myQuery = "select name from ispvn where id = "+ vIspvn.elementAt(i);
  	 myQuery += " order by id";
//System.out.println(myQuery);

  	 myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

      if (myDbBean.loadData()) {
    	  while (myDbBean.getNextRecord()){
	  			 resultV.addElement(myDbBean.getColumn("NAME"));
	     }
      }
   }
   try{
          myDbBean.takeDown();
   }catch(Exception e){
          System.out.println("Error Connection:    "+e);
   }
   return resultV;
  }


	/********************************************************************
	* Method:  Vector getISPVN_ID_NameList()
	*
	* Purpose:  This method returns a list of name from ispvn by
	*           the given Ispvn.
	*
	* Note:     Needs to call setOperatorGroupID prior to this method.
	********************************************************************/

	public Vector getISPVN_ID_NameList(int groupId) throws Exception
	{
		Vector resultV = new Vector();

		setOperatorGroupID(groupId);
		myQuery = "";
		myQuery = "select a.ispvn_id, b.name from operator_ispvn a, ispvn b "+
					 "where a.operator_group_id = "+groupId+" And b.id=a.ispvn_id";

		myDbBean.setMyQuery(myQuery);
		myDbBean.execute_sql(myQuery);

		if (myDbBean.loadData()) {
		while (myDbBean.getNextRecord()) {
			String[] colData = new String[2];
			colData[0] = myDbBean.getColumn(1);
			colData[1] = myDbBean.getColumn(2);
			resultV.addElement(colData);
		}
		}
		try{
			myDbBean.takeDown();
		}catch(Exception e){
			System.out.println("Error Connection:    "+e);
		}

		return resultV;
	}

	public static void main(String[] args)
	{
		UserAuthentication tAuth = new UserAuthentication();
		try {
			int gid = tAuth.authenticate(args[0], args[1]);
System.out.println("GroupId: "+gid);
System.out.println("OperatorId: "+tAuth.getOperatorID());
			Vector tv = tAuth.getISPVN_ID_NameList(gid);
			for(int i=0; i < tv.size(); i++)
			{
				String[] colData = (String[])(tv.elementAt(i));
				System.out.println(colData[0]+",  "+colData[1]);
			}
		} catch (Exception e) { System.out.println(e.toString()); }
	}
}
