
// DBApp.java
/***************************************************************************************/
// Program Name: DBApp.java
//
//
//
// Date Written: November, 2000
//
// Programmer: H. Huang
//
// Purpose:  To query database and retrieve data
//
//
//
// Update(s):  July, 2001:  H. Huang
//             1) in catch(Exception e) of  execute_sql(String sqlString):
//                Added:   ConnectionPool.releaseConnection( myConn );  //
//             2) cleanup():
//   	              myResultSet.close();
//    	            myResultSet = null;
//                  stmt.close();
//                  stmt = null;
//
//
/**************************************************************************************/

package portal.bean;

import java.sql.*;
import java.io.*;

public class DBApp extends ConManager
{
  private String myQuery ;
  private int noColumns ;
  private ResultSet myResultSet = null;
  private Statement stmt = null;

  private boolean maxRowState = false;
  private int maxRows = 0;


  public DBApp() {
      super();
  	  myQuery = "";
      noColumns = 0;
  }


/********************************************************************
* Method:   setMyQuery(String s)
*
* Purpose:  This method sets myquery.
*********************************************************************/

 public void setMyQuery(String s){
		myQuery = s;
  }


/********************************************************************
* Method:  String getMyQuery(String s)
*
* Purpose:  This method returns myquery.
*********************************************************************/

 public String getMyQuery(String s){
	  return myQuery;
  }


/********************************************************************
* Method:   setMyResultSet(ResultSet r)
*
* Purpose:  This method sets myResultSet.
*********************************************************************/

 public void setMyResultSet(ResultSet r){
		myResultSet = r;
  }



/********************************************************************
* Method:   ResultSet getMyResultSet()
*
* Purpose:  This method gets myResultSet.
*********************************************************************/

 public ResultSet getMyResultSet(){
		return myResultSet;
 }


/********************************************************************
* Method:   setStmt(String s)
*
* Purpose:  This method sets stmt.
*********************************************************************/

 public void setStmt(Statement s){
		stmt = s;
  }


/********************************************************************
* Method:  Statement getStmt()
*
* Purpose:  This method gets stmt.
*********************************************************************/

 public Statement getStmt(){
		 return stmt;
  }


/********************************************************************
* Method:   void setMaxRowState(boolean b, int i)
*
* Purpose:  This method sets maxRowState and maxRows .
*
* @param   b  set maxRowState true or false
* @param   i  the max rowNum
*********************************************************************/

  public void setMaxRowState(boolean b, int i){
	   maxRowState = b;
	   maxRows = i;
	}


/********************************************************************
* Method:   void cleanMaxRowState( )
*
* Purpose:  This method clean maxRowState and maxRows .
*
*********************************************************************/

  public void cleanMaxRowState(){
	   maxRowState = false;
	   maxRows = 0;
	}


/********************************************************************
* Method:   getNextRecord()
*
* Purpose:  This method gets next record.
*********************************************************************/

  public boolean getNextRecord() throws Exception
  {
    return myResultSet.next();
  }



/********************************************************************
* Method: boolean previousRecord()
*
* Purpose:  This method moves cursor to the previous row.
*********************************************************************/

   /*public boolean previousRecord()     throws Exception
  {
     return  myResultSet.previous() ;
   }*/


/********************************************************************
* Method: boolean firstRecord()
*
* Purpose:  This method moves cursor to the first row.
*********************************************************************/

   /*public boolean firstRecord()     throws Exception
  {
     return  myResultSet.first() ;
   }*/


/********************************************************************
* Method:   String getColumn( String inCol)
*
* Purpose:  This method gets field(String) by field's name in a record.
*********************************************************************/

  public String getColumn( String inCol) throws Exception
  {
    return myResultSet.getString(inCol);
  }

/********************************************************************
* Method:  String getColumn( int inCol)
*
* Purpose:  This method gets field(String) by column's index in a record.
*********************************************************************/

 public String getColumn( int inCol) throws Exception
  {
    return myResultSet.getString(inCol);
  }


/********************************************************************
* Method:  int getColumn( String inCol)
*
* Purpose:  This method gets field(int) by column's name in a record.
*********************************************************************/

  public int getColumnInt( String inCol) throws Exception
  {
    return myResultSet.getInt(inCol);
  }


/********************************************************************
* Method:  long getColumn( String inCol)
*
* Purpose:  This method gets field(int) by column's name in a record.
*********************************************************************/

  public long getColumnLong( int inCol) throws Exception
  {
    return myResultSet.getLong(inCol);
  }



/********************************************************************
* Method:  int getColumnInt( int inCol)
*
* Purpose:  This method gets field(int) by column's index in a record.
*********************************************************************/

  public int getColumnInt(int inCol) throws Exception
  {
    return myResultSet.getInt(inCol);
  }


/********************************************************************
* Method:  double getColumnDouble( String inCol)
*
* Purpose:  This method gets field(int) by column's index in a record.
*********************************************************************/

  public double getColumnDouble(String inCol) throws Exception
  {
    return myResultSet.getDouble(inCol);
  }



/********************************************************************
* Method:  double getColumnDouble( int inCol)
*
* Purpose:  This method gets field(int) by column's index in a record.
*********************************************************************/

  public double getColumnDouble(int inCol) throws Exception
  {
    return myResultSet.getDouble(inCol);
  }


/********************************************************************
* Method:  boolean getRecords()
*
* Purpose:  This method executes query.
*********************************************************************/

  public boolean getRecords()     throws Exception
  {
   // String myQuery = myEmpSQL;
       stmt = myConn.createStatement();
     myResultSet = stmt.executeQuery(myQuery);
     return (myResultSet != null);
   }


/********************************************************************
* Method: boolean execute_sql(String sqlString)
*
* Purpose:  This method executes query.
*********************************************************************/

//   public void execute_sql(String sqlString)     throws Exception
  public void execute_sql(String sqlString)
  {
  	try{
      myResultSet = null;
      myQuery = sqlString;
      stmt = myConn.createStatement();
      // set maxRows
      if(maxRowState){
        stmt.setMaxRows(maxRows);
        cleanMaxRowState();
      }
      myResultSet = stmt.executeQuery(myQuery);
     // return (myResultSet != null);
    }catch(Exception e) {
    	 ConnectionPool.releaseConnection( myConn );
    	 System.out.println("Error in DBapp.execute_sql, but release connnection:" + e);
   	}
   }

/********************************************************************
* Method: boolean loadData()
*
* Purpose:  This method executes query.
*********************************************************************/

   public boolean loadData()     throws Exception
  {
     return (myResultSet != null);
   }


/********************************************************************
* Method: int getNoColumn()
*
* Purpose:  This method returns the number of collumns in the result
*           set.
*********************************************************************/

   public int getNoColumn()  throws Exception
   {
     ResultSetMetaData metaData = myResultSet.getMetaData();
     noColumns =  metaData.getColumnCount();
     return noColumns;
   }


/********************************************************************
* Method: void cleanup()
*
* Purpose:  This method closes stmt.
*********************************************************************/
  //public void cleanup() throws Exception
    public void cleanup()
   {
   	  System.out.println("close myResultSet & Statement" );		//hhuang
   	  try {
    	  myResultSet.close();
    	  myResultSet = null;
        stmt.close();
        stmt = null;
        System.out.println("myResultSet = " + myResultSet+"\nstmt = "+stmt);
    	}catch(Exception e) { System.out.println("Error in DBapp.cleanup. " + e); }
   }

}
