/*
 Program Name: DbUtil.java
 Author : Nandeesh
 Date : Feb, 2001
 Purpose:  To implement the database related functionality like Select,Insert,Update,Delete
           and transaction operations for commiting and rolling back.
*/

package portal.bean;

import portal.bean.*;

import java.sql.*;
import java.io.*;
import java.util.*;

	/**
		* To implement the database related functionality like Select,Insert,Update,Delete
        * and transaction operations for commiting and rolling back.
     */


public class DbUtil {
	private Connection  _myConn;
	private ConnectionPool cPool;
	private boolean _inTransaction;

	public DbUtil() {
		_myConn = null;
		cPool = new ConnectionPool();
		_inTransaction = false;
		}

	/**
		* Indicates  the begining of transaction
		* @return   void  Message will be displyed/printed in case of exception
		*
		* @param    Nothing
	 */
	public void beginTrans() {
		  try {
			System.out.println("Trying to get connection before in BeginTrans");
		   _myConn = cPool.getConnection();

		   _myConn.setAutoCommit(false);
		   _inTransaction = true;
		   }

          catch (Exception ex) {
              System.out.println("Exception during BeginTrans/Connection,setAutoCommit:    " + ex);
			}
	  }

		/**
			* Rolls back all the changes to the database since last commit/rollback
			* @return   void  Message will be displyed/printed in case of exception
			*
			* @param    Nothing
		 */

	public void rollback() {
			try {
			   _myConn.rollback();
				cPool.releaseConnection(_myConn);
				System.out.println(" rolled back ..");
				_myConn = null;
				_inTransaction = false;
			   }
			catch (Exception ex) {
				System.out.println("Exception during rollback/releaseConnection:    " + ex);
			   }
	 }

		/**
			* Commits all the changes to the database since last commit/rollback
			* @return   void  Message will be displyed/printed in case of exception
			*
			* @param    Nothing
		 */

	public void commitTrans() {
		try {
			_myConn.commit();
			cPool.releaseConnection(_myConn);
					System.out.println(" commited ..");
			_myConn = null;
			_inTransaction = false;
				}
			catch (Exception ex) {
				System.out.println("Exception during commit:    " + ex);
			   }
	}


		/**
			* Executes the non-select queries like INSERT,UPDATE,DELETE..
			* @return   int The number of rows affected.
			*
			* @param    _pSql the SQL string to be Updated.
		 */

		public int ExecuteUpdateQuery(String _pSql)  {

				Statement _stmt ;
				int _queryResult = 0;

				try {

					 if (_myConn == null || _myConn.isClosed()) {
					     _myConn = cPool.getConnection();
					     //System.out.println("was null.. and  cPool.getConnection() executed");
					    }
					  else
					     //System.out.println("did not create new connection...");
					     _myConn.setAutoCommit(false);

					  _stmt = _myConn.createStatement();
					  System.out.println("sql running..." + _pSql.toString() );
					 // System.out.println("in ExecuteUpdateQuery ..myConn info" + _myConn );
					  _queryResult = _stmt.executeUpdate(_pSql.toString());
					  System.out.println(" NO DB error.... executed....");
					  _stmt.close();
					}
					catch (Exception ex) {
						System.out.println("Error Connection:    " + ex);
					}
					finally {
						//cPool.releaseConnection(_myConn);
						if (!_inTransaction) {
									cPool.releaseConnection(_myConn);
									_myConn = null;
									//System.out.println("\n NOT in  transaction..so connection  Released..");
							}
						//else
							//System.out.println("\n in the transaction..so didnot release");
						return _queryResult;
					}
		}

		/**
			* Executes the SELECT query
			* @return   Vector of string array with number of columns as in _pCols
			*
			* @param    _pCols the number of columns to be selected as part of SELECT query
		 */

		 public Vector ExecuteSelectQuery(String _pSql, int _pCols) {

			 	ResultSet _myResultSet ;
			 	Statement _stmt ;
				Connection  _mySelectConn = null;
				Vector vOut = new Vector(10, 10);
				String[] _resultRowCols;

				//select TO_CHAR(SYSDATE, 'Month DD,YYYY HH24:MI:SS') "Now" FROM dual;
				//select TO_CHAR(timestamp, 'Month DD,YYYY HH24:MI:SS')  FROM transaction
				System.out.println("Entered ExecuteSelectQuery()");

				 try {
						_mySelectConn = cPool.getConnection();
						_stmt = _mySelectConn.createStatement();
						System.out.println("query-->" + _pSql.toString());
						_myResultSet = _stmt.executeQuery(_pSql.toString());

						while(_myResultSet.next())
						{
							_resultRowCols = new String[_pCols];
							for(int i=1; i <= _pCols; i++)
								_resultRowCols[i-1] = _myResultSet.getString(i);
							vOut.addElement(_resultRowCols);
						}
						_myResultSet.close();
						_stmt.close();
					  }
					catch (Exception ex) {
						System.out.println("Error Connection:    " + ex);
					    }
					finally {cPool.releaseConnection(_mySelectConn);
							System.out.println("Released connection after ExecuteSelectQuery" );
	             				  return vOut;
							}
			}
	}