

/***************************************************************************************/
// File: IMSInvoice.java
// Date: 02/07/2001
//
// Author: Amelia M. Chan
//
// 2/26/2001  move myDbBean.makeConnection() to DbBean.java.
//            myDbBean.takeDown()
//
// Acronym: used when the title of a method gets too long.
//
// OID = OrderID
// OIDList = OrderID List
// AID = AccountID
// AIDList = AccountID List
//
/**************************************************************************************/
package portal.invoice;


import java.sql.*;
import java.io.*;
import java.util.*;
import java.sql.Date;
import portal.bean.*;

public class IMSInvoiceBean implements IMSInvoice

{
  private int ISPvnID;
  private int accountID;
  private int orderID;
  private String myQuery;
  private DbBean myDbBean;
  //global to store the queryResultSet for paging purpose.
  private Vector queryResult;

  public IMSInvoiceBean()
 {
	  ISPvnID = 0;
	  accountID = 0;
	  orderID = 0;
	  myQuery = "";

      myDbBean = new DbBean();
      queryResult = new Vector();
 }

  public void setISPvnID(int num) {
     this.ISPvnID = num;
  }

  public int getISPvnID() {
     return ISPvnID;
  }


  public void setAccountID(int num) {
     this.accountID = num;
  }

  public int getAccountID(){
	    return accountID;
  }


  public void setOrderID(int num) {
       this.orderID = num;
  }

  public int getOrderID(){
  	    return orderID;
  }


 public void setQueryResult(Vector qResult) {
       this.queryResult = qResult;
  }

  public Vector getQueryResult(){
  	    return queryResult;
  }

 /********************************************************************
  *
  *      Account
  *
  ********************************************************************/

/********************************************************************
* Method:  Vector getAIDListByISPVN()
*
* Purpose:  This method returns a list id from accounts by the given ispvnID.
* return:  a Vector includes one or more ispvn_id.
*
********************************************************************/
public Vector getAIDListByISPVN(int ispvnID) throws Exception
{
     Vector resultV = new Vector();
     myQuery = "";
     myQuery="select id from account where ispvn_id = "+ ispvnID + " order by id";

     myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
	    while (myDbBean.getNextRecord())
	    {
	        Integer num = new Integer (myDbBean.getColumnInt("ID"));
	        resultV.addElement(num);
        }
     }
   	setQueryResult(resultV);  // save it for paging.
    try{
         myDbBean.takeDown();
    }catch(Exception e){
         System.out.println("Error Connection:    "+e);
    }

    return resultV;
}



/********************************************************************
* Method:  Vector getAIDList(int ispvnID, String criteria)
*
* Purpose:  This method returns a list of id from account by the given ispvnID.
* return:  a Vector includes one or more ispvn_id.
*
********************************************************************/

public Vector getAIDListByISPVNCriteria (int ispvnID, String criteria) throws Exception
{
     Vector resultV = new Vector();
     myQuery = "";
     myQuery = "select id from account where ispvn_id = "+ ispvnID;
     myQuery += " AND " + criteria;

     myDbBean.setMyQuery(myQuery);
	 myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
	    while (myDbBean.getNextRecord())
	    {
	        Account anAccount = new Account();

		    anAccount.setAnAccount(myDbBean.getColumnInt("ID"),
						           myDbBean.getColumn("COMPANY"),
						           myDbBean.getColumn("SURNAME"),
						           myDbBean.getColumn("FIRSTNAME"));
			resultV.addElement(anAccount);

        }
     }
   	setQueryResult(resultV);  // save it for paging.
    try{
         myDbBean.takeDown();
    }catch(Exception e){
         System.out.println("Error Connection:    "+e);
    }
    return resultV;
}


/********************************************************************
* Method:  Vector getAccountInformation()
*
* Purpose:  This method returns a list of selected account information by the given AccountID.
* return:  a Vector includes one or more ispvn_id.
*
* return: a vector with one or no element (id, company, surname and firstname)
*         no row will be return if not found.
********************************************************************/

public Vector getAccountInformation(int accountID) throws Exception
{
     Vector resultV = new Vector();
     myQuery = "";
     myQuery="select id, company, surname, firstname from account where id = "+ accountID;

     myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
	    while (myDbBean.getNextRecord())
	    {
			Account anAccount = new Account();

		    anAccount.setAnAccount(myDbBean.getColumnInt("ID"),
			                          myDbBean.getColumn("COMPANY"),
			                          myDbBean.getColumn("SURNAME"),
			                          myDbBean.getColumn("FIRSTNAME"));
			resultV.addElement(anAccount);
        }
     }
 	 setQueryResult(resultV);  // save it for paging.
     try{
          myDbBean.takeDown();
     }catch(Exception e){
          System.out.println("Error Connection:    "+e);
     }
     return resultV;
}


/********************************************************************
* Method:  int getIspvnByAccountID(int accountID)
*
* Purpose:  This method returns ispvn_id from account by the given accountID.
*           will be called by generateInvoice(). 3/16/2001
********************************************************************/

public int getIspvnByAccountID(int accountID) throws Exception
{
     myQuery = "";
     myQuery="select ispvn_id from account where id = "+ accountID;

     myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
	    while (myDbBean.getNextRecord())
	    {
		   ISPvnID = myDbBean.getColumnInt("ISPVN_ID");
        }
     }
     return ISPvnID;
}



 /********************************************************************
  *
  *      Orders
  *
  ********************************************************************/

/********************************************************************
 update orders set sum_total = 0 where id =
 ex: select * from orders where id = 381;
     update orders set sum_total = 0 where id = 1017;
 ********************************************************************/


public void initOrdersSumTotal(Vector vOrderID) throws Exception
{
    for (int i=0; i < vOrderID.size(); i++ )
 	{
         myQuery = "";
         myQuery = "update Orders set sum_total = 0";
         myQuery += " where id = " + vOrderID.elementAt(i);
         myDbBean.setMyQuery(myQuery);
         myDbBean.execute_sql(myQuery);
    }
}

/********************************************************************
   ********************************************************************/


public void updateAnOrdersSumTotal(int OrderID, String bStart, String bEnd) throws Exception
{

     myQuery = "";
     myQuery= "update Orders set sum_total = (select sum(total) from order_line";
     myQuery += " where order_id = " + OrderID + ")";
     myQuery += " where id = " + OrderID;

     //If bStart is empty, select eveything by skipping appending this part of code.
     if( ! bStart.equals("") ){
         myQuery += " and billing_period_start >= '" + bStart + "'";
     }

     //If bEnd is empty, select eveything by skipping appending this part of code.
     if( ! bEnd.equals("") ){
 	  myQuery += " and billing_period_end <= '" + bEnd + "'";
     }
	 myDbBean.setMyQuery(myQuery);
   	 myDbBean.execute_sql(myQuery);
}


/********************************************************************
3/19/2001 removed takeDown.
3/19/2001 if there is no record in order_item_details table, will cause the following error
java.sql.SQLException: ORA-01407: cannot update("IMS"."ORDERS.sum_total to NULL;

So, need to check is there record in the order_item_details table.

     myQuery = "";
     myQuery= "update Orders set sum_total = (select sum(total) from order_line";
     myQuery += " where order_id = " + vOrderID.elementAt(i) + ")";
     myQuery += " where id = " + vOrderID.elementAt(i);

// noColumn = 6.
//System.out.print("%%%%%total column="); System.out.println(myDbBean.getNoColumn());

   ********************************************************************/

public void updateOrdersSumTotal(Vector vOrderID) throws Exception
{
    for (int i=0; i < vOrderID.size(); i++ )
 	{
         myQuery = "";
	     myQuery= "select * from order_line where order_id = " + vOrderID.elementAt(i);
//System.out.println(myQuery);
         myDbBean.setMyQuery(myQuery);
         //myDbBean.initResultSet();
   	     myDbBean.execute_sql(myQuery);

   	     myQuery = "";
   	     if (myDbBean.loadData())
	     {
 	         if (myDbBean.getNextRecord())
  	         {
//System.out.println("getNextRecord is true-------");
                myQuery= "update Orders set sum_total = (select sum(total) from order_line";
		        myQuery += " where order_id = " + vOrderID.elementAt(i) + ")";
                myQuery += " where id = " + vOrderID.elementAt(i);
 	 	     }
 	 	     else
 	 	     {
//System.out.println("getNextRecord is false -------");
                myQuery= "update Orders set sum_total = 0 where id = " + vOrderID.elementAt(i);
	         }
	         myDbBean.setMyQuery(myQuery);
	         //myDbBean.initResultSet();
   	         myDbBean.execute_sql(myQuery);
  	     }
     }
//System.out.print("out updateOrdersSumTotal--------");
}


   /********************************************************************
   * Method:  Vector getOIDListByISPVN() (03/01/2001)
   *
   * Purpose: This method performs two queries and returns a list of
   *          accountID list by the given ispvnID.
   *
   * Sql:	  "select id from account where ispvn_id = 1";
   *
   *          "select id from Orders where account_id = X";
   *
   * call public Vector getOIDListByAIdList(int[] accoutIDlist)
   ******************************************************************/

public Vector getOIDListByISPVN(int ispvnID) throws Exception
{
     Vector resultV = new Vector();
     try{
		 resultV = getAIDListByISPVN(ispvnID);
   	 }
   	 catch (Exception ex) {
   		   System.out.println("Error Connection:    "+ex);
     }
     setQueryResult(getOIDListByAIDList(resultV));  // save it for paging.
     return getQueryResult();
}


   /********************************************************************
   * Method:  Vector getOrdersByISPVN() (03/01/2001)
   *
   * Purpose: This method performs two queries and returns a list of
   *          accountID list by the given ispvnID.
   *
   * Sql:	  "select id from account where ispvn_id = 1";
   *
   *          "select * from Orders where account_id = X";
   *
   * call public Vector getOrdersByAIdList(int[] accoutIDlist)
   ******************************************************************/

public Vector getOrdersByISPVN(int ispvnID) throws Exception
{
      Vector resultV = new Vector();
      try{
 		 resultV = getAIDListByISPVN(ispvnID);
    	 }
    	 catch (Exception ex) {
    		   System.out.println("Error Connection:    "+ex);
      }
      setQueryResult(getOrdersByAIDListDates(resultV, "", ""));  // save it for paging.
     return getQueryResult();
}



/********************************************************************
* Method:  Vector getOIDListByAIdList (Vector vAccoutID) (03/01/2001)
*
* Purpose:  This method returns a list of OrderIDList by the given accountIDList
* return:  A Vector includes one or more orderIDList.
*
*  returns orderIDList by "select id from Orders where account_id =
*******************************************************************/

public Vector getOIDListByAIDList(Vector vAccoutID) throws Exception
{
   Vector resultV = new Vector();

   for (int i=0; i < vAccoutID.size(); i++ )
   {
  	 myQuery = "";
  	 myQuery = "select id from Orders where account_id  = "+ vAccoutID.elementAt(i);
  	 myQuery += " order by id";

  	 myDbBean.setMyQuery(myQuery);
//System.out.println(myQuery);
	 myDbBean.execute_sql(myQuery);

  	 if (myDbBean.loadData()) {

  	     while (myDbBean.getNextRecord())
  	     {
			 Integer num = new Integer (myDbBean.getColumnInt("ID"));
			 resultV.addElement(num);
 	 	 }
  	 }
   }
   setQueryResult(resultV);  // save it for paging.

    try{
   	     myDbBean.takeDown();
   	}catch(Exception e){
   	       System.out.println("Error Connection:    "+e);
	}
   return resultV;
}




/********************************************************************
* Method:  Vector getOIDListByAIDListDates
* input:   Vector vAccoutID, String bStart, String bEnd (03/01/2001)
*
* Purpose:  loop each element of vAccountID
*           to retrieve selected rows from orders.
*
* 1) 3/19/2001 If bStart is empty, select eveything by not appending this part of code.
* 2) 3/19/2001 If bEnd is empty, select eveything by not appending this part of code.
******************************************************************/

public Vector getOIDListByAIDListDates(Vector vAccoutID, String bStart, String bEnd) throws Exception
{
   Vector resultV = new Vector();
   for (int i=0; i < vAccoutID.size(); i++ )
   {
      myQuery="select id from orders where account_id = "+ vAccoutID.elementAt(i);

      //If bStart is empty, select eveything by skipping appending this part of code.
      if( ! bStart.equals("") ){
        myQuery += " and billing_period_start >= '" + bStart + "'";
      }

      //If bEnd is empty, select eveything by skipping appending this part of code.
      if( ! bEnd.equals("") ){
	  myQuery += " and billing_period_end <= '" + bEnd + "'";
      }

	  myQuery += " order by id";

  	  myDbBean.setMyQuery(myQuery);
	  myDbBean.execute_sql(myQuery);

  	 if (myDbBean.loadData())
  	 {
  	     while (myDbBean.getNextRecord())
  	     {
			 Integer num = new Integer (myDbBean.getColumnInt("ID"));
			 resultV.addElement(num);
 	 	 }
  	 }
   }
   setQueryResult(resultV);  // save it for paging.
   /*
    try{
   	     myDbBean.takeDown();
   	}catch(Exception e){
   	        System.out.println("Error Connection:    "+e);
	}
   */
   return resultV;
}



/********************************************************************
* Method:  Vector getOrdersByAIDListDates
*          (Vector vAccoutID, String bStart, String bEnd) (03/01/2001)
*
* Purpose:  This method returns a list of OrderIDList by the given accountIDList
* return:  A Vector includes one or more orderIDList.
*
*  returns orderIDList by "select id from Orders where account_id =
*******************************************************************/

public Vector getOrdersByAIDListDates(Vector vAccoutID, String bStart, String bEnd) throws Exception
{
	// update sum_total at the give id.
   Vector vOrderID = new Vector();

   // returns vOrderID between bStart and bEnd.
   vOrderID = getOIDListByAIDListDates(vAccoutID, bStart, bEnd);
/*
System.out.print("getOIDListByAIDListDates. vOrderID = ");System.out.println(vOrderID.size());

 	  for (int i=0; i < vOrderID.size(); i++ )
 	  {
 	 	 System.out.println(vOrderID.elementAt(i));
 	  }

System.out.println("before updateOrdersSumTotal") ;
*/
   updateOrdersSumTotal(vOrderID);

   Vector resultV = new Vector();
   for (int i=0; i < vOrderID.size(); i++ )
   {
       myQuery="select * from orders where id = "+ vOrderID.elementAt(i);
	   myQuery += " order by id";
//System.out.println(myQuery);

       myDbBean.setMyQuery(myQuery);
  	   myDbBean.execute_sql(myQuery);

        if (myDbBean.loadData())
       {
   	      while (myDbBean.getNextRecord())
   	      {
		      Order anOrder = new Order();

		      anOrder.setAnOrder(myDbBean.getColumnInt("ID"),
				               myDbBean.getColumn("STATE"),
				               myDbBean.getColumnDate("BILLING_PERIOD_START"),
				               myDbBean.getColumnDate("BILLING_PERIOD_END"),
				               myDbBean.getColumnDate("INVOICE_GENERATED"),
   			                   myDbBean.getColumnInt("ACCOUNT_ID"),
   			                   myDbBean.getColumnDouble("SUM_VAT_TAX"),
   			                   myDbBean.getColumnDouble("SUM_PRICES"),
   			                   myDbBean.getColumnDouble("SUM_TOTAL"));

              resultV.addElement(anOrder);
           }
       }
    }
    setQueryResult(resultV);  // save it for paging.
    try{
    	     myDbBean.takeDown();
    	}catch(Exception e){
    	        System.out.println("Error Connection:    "+e);
 	}
    return resultV;
 }


 /********************************************************************
   * Method:  Vector getOrders()
   * Note:     Needs
   *************************************************************/

public Order getOrders(int orderID) throws Exception
{
System.out.println("begin getOrders ---");

     Order anOrder = new Order();

     myQuery = "";
     myQuery="select * from orders where id = "+ orderID;

 System.out.println(myQuery);
     myDbBean.setMyQuery(myQuery);
   	 myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {
   	     while (myDbBean.getNextRecord())
   	     {
			 anOrder.setAnOrder(myDbBean.getColumnInt("ID"),
				                myDbBean.getColumn("STATE"),
				                myDbBean.getColumnDate("BILLING_PERIOD_START"),
				                myDbBean.getColumnDate("BILLING_PERIOD_END"),
				                myDbBean.getColumnDate("INVOICE_GENERATED"),
   			                    myDbBean.getColumnInt("ACCOUNT_ID"),
   			                    myDbBean.getColumnDouble("SUM_VAT_TAX"),
   			                    myDbBean.getColumnDouble("SUM_PRICES"),
   			                    myDbBean.getColumnDouble("SUM_TOTAL"));
   		 }
    }
    try{
   	     myDbBean.takeDown();
   	}catch(Exception e){
   	        System.out.println("Error Connection:    "+e);
	}

    return anOrder;
}



 /********************************************************************
  * Method:  void generateInvoice(int orderID)
  *
  * output:  Table order
             1) check/update the order by the giving ordeID.
  *            - set state to be 'E'.
               - sum up the sum_total;
  *
             Table Transaction
  *          1) put 'INV' on the associated Transaction Table.
  *
  *          2) set reference_id = orderID
  *
  *          3) Amount = Orders.sum_total;
  *
  *******************************************************************/

public void generateInvoice(int orderID) throws Exception
{
	String inv = "INV";
 	myQuery = "";
    myQuery= "update Orders set state = 'E' where id = "+ orderID;

	myDbBean.setMyQuery(myQuery);
    myDbBean.execute_sql(myQuery);

    updateAnOrdersSumTotal(orderID, "", "");

    Order anOrder = new Order();
    anOrder = getOrders(orderID);

    //int ispvn_id = getIspvnByAccountID(anOrder.getAccountID());
  myQuery = "";
  myQuery += "INSERT INTO transaction(id,trans_type,account_id,ispvn_id,amount,reference_id,timestamp)";
  myQuery += " VALUES (transaction_seq.NEXTVAL," + "'" + inv + "'" + "," + anOrder.getAccountID() + ",";
  myQuery += getIspvnByAccountID(anOrder.getAccountID()) + "," + anOrder.getSumTotal() + "," + orderID + "," + "SYSDATE" + ")";

    myDbBean.setMyQuery(myQuery);
	myDbBean.execute_sql(myQuery);

   myQuery = "";
   myQuery += "UPDATE balance set amount = " + anOrder.getSumTotal();
   myQuery += " WHERE account_id = " + anOrder.getAccountID();

   myDbBean.setMyQuery(myQuery);
   myDbBean.execute_sql(myQuery);
   try{
      	 myDbBean.takeDown();
   }catch(Exception e){
      	        System.out.println("Error Connection:    "+e);
   }
}



 /********************************************************************
  * Method:  void deleteTran(int orderID)
  * has sql statement error! (3/12/2001)
  *******************************************************************/

public void deleteTran(int tranID) throws Exception
{
    myQuery = "";
    myQuery = "execute transactions.deleteTransaction(";
    myQuery += tranID + ")";
//System.out.println(myQuery);

    myDbBean.setMyQuery(myQuery);
    myDbBean.execute_sql(myQuery);

    try{
	   	 myDbBean.takeDown();
	}catch(Exception e){
	   	        System.out.println("Error Connection:    "+e);
    }

//System.out.println("out generateInvoice ---");
}



   /********************************************************************
    *
    *      Order_item_details
    *
    ********************************************************************/



 /********************************************************************
 * Vector getOrderDetails(int[] OIdList, String bStart, String bEnd)
 *
 *          loop each element in the list to execute query
 *          append each resultSet to be the global big resultSet.
 *
 * 1) 3/19/2001 If bStart is empty, select eveything by not appending this part of code.
 * 2) 3/19/2001 If bEnd is empty, select eveything by not appending this part of code.
 *
 *******************************************************************/

public Vector getOrderDetailsByOIDListDates (Vector vOrderID, String bStart, String bEnd) throws Exception
{
System.out.println("begin getOrderDetails ---");

  	Vector resultV = new Vector();

    for (int i=0; i < vOrderID.size(); i++ )
	{
  	 myQuery = "";
   	 myQuery = "select * from order_item_details where order_id = "+ vOrderID.elementAt(i);

	 //If bStart is empty, select eveything by skipping appending this part of code.
	 if( ! bStart.equals("") ){
		 myQuery += " and timestamp >= '" + bStart + "'";
     }
     //If bEnd is empty, select eveything by skipping appending this part of code.
	 if( ! bEnd.equals("") ){
	     myQuery += " and timestamp <= '" + bEnd + "'";
     }
	 myQuery += " order by timestamp";
System.out.println(myQuery);

  	 myDbBean.setMyQuery(myQuery);
     myDbBean.execute_sql(myQuery);

     if (myDbBean.loadData()) {

  	     while (myDbBean.getNextRecord())
  	     {
  	 	     OrderItemDetails anOID = new OrderItemDetails();
  	 	     anOID.setAnOrderItemDetails(myDbBean.getColumnDate("TIMESTAMP"),
  	 	       	 	                        myDbBean.getColumnInt("ORDER_ID"),
  	 	                                    myDbBean.getColumnInt("ITEM_ID"),
  	 				                        myDbBean.getColumnInt("QUANTITY"),
  	 				                        myDbBean.getColumnInt("AMOUNT"),
  	    			                        myDbBean.getColumn("DESCRIPTION"));
  	 	     resultV.addElement(anOID);
  	 	 }
  	  }
  	}
System.out.print("resultV:="); System.out.println(resultV.size());

  	setQueryResult(resultV);  // save it for paging.

	try{
	   	 myDbBean.takeDown();
	}  catch (Exception ex) {
	   	System.out.println("Error Connection:    "+ex);
    }

	 return resultV;
 }



 /********************************************************************
 * Method:  Vector pagingQResults(int start, int end)
 *
 * return: a subset of global queryResult.
 * 3/19 passing qResult as one para is bad.  think about qResult.size = 20000.
 ********************************************************************/

 public Vector pagingQResults(int start, int end) throws Exception
 {
System.out.println("in VectorPaging ---");

    Vector resultV = new Vector();
    Vector resultX = new Vector();
    resultX = getQueryResult();
    for (int i=start; i <= end; i++ ) {
		if (i < resultX.size()){
		 resultV.addElement(resultX.elementAt(i));
	    }
	    else   // exit from the for loop.
	       i = (end + 1);
    }
System.out.println("out  VectorPaging ---");
    return resultV;
  }


   /********************************************************************
    * Method:  Vector reInvoicing(Vector vAccoutID, String bStart, String bEnd)
    *
    * returns order_item_details by the given accountId list and date range.
    * make sure that the state in Orders table is set to be 'E'.
    * 3/12 is generateInvoice work right.  this step should be spared.
    * 3/16 re-testing generateInvoice(int orderId) work.
    *
    *******************************************************************/

  public Vector reInvoicing(Vector vAccoutID, String bStart, String bEnd) throws Exception
  {
  	  Vector vOrderID = new Vector();
  	  vOrderID = getOIDListByAIDListDates(vAccoutID, bStart, bEnd);

  	  Vector resultV = new Vector();
      resultV = getOrderDetailsByOIDListDates(vOrderID, bStart, bEnd);

      setQueryResult(resultV);  // save it for paging.
      return resultV;
  }



 /********************************************************************
  * Method:  Vector hotBilling(int orderID)
  * steps:
  * 0) store the value of billing_period_end.
  * 1) set state = 'E'.
  * 2) set billing_period_end = SYSDATE.
  * 3) insert a new record (order_seq.NEXTVAL, 'A', sysdate, billing_period_end.
  * 4) set 'inv' oin the transaction table.
  * 5) update Amount in Balance table with order.sum_total
  *******************************************************************/

  public void hotBilling(int orderID) throws Exception
  {
		String inv = "INV";

		updateAnOrdersSumTotal(orderID, "", "");

        Order anOrder = new Order();
        //get the whole record of the given orderID.
        anOrder = getOrders(orderID);

        // set state = 'E'.
 	    myQuery = "";
  	    myQuery = "UPDATE orders SET State = '" + "E" + "'";
  	    myQuery += ", billing_period_end = SYSDATE";
  	    myQuery += " WHERE id  = " + orderID;
 System.out.println(myQuery);
 		myDbBean.setMyQuery(myQuery);
        myDbBean.execute_sql(myQuery);


        //" and timestamp  < TO_DATE('" + endDate + "', 'MM/DD/YYYY') + 1 ";
		// insert a new record
        myQuery = "";
	    myQuery = "INSERT INTO orders (id, state, billing_period_start, billing_period_end,";
	    myQuery += " invoice_generated, account_id, sum_vat_tax, sum_prices, sum_total)";
	    myQuery += " VALUES (order_seq.NEXTVAL, '" + "A" + "'" + ", SYSDATE,";
	    myQuery += " TO_DATE('" + anOrder.getBillingPeriodEnd() +"', 'YYYY-MM-DD')" + ", SYSDATE, ";
	    myQuery += anOrder.getAccountID() + "," + anOrder.getSumVatTax() + ",";
	    myQuery += anOrder.getSumPrices() + "," + anOrder.getSumTotal() + ")";
System.out.println(myQuery);
		myDbBean.setMyQuery(myQuery);
		myDbBean.execute_sql(myQuery);

		// update the transaction table
	    myQuery = "";
		myQuery += "UPDATE transaction set trans_type = '" + inv + "'";
		myQuery += " WHERE reference_id  = " + orderID;
System.out.println(myQuery);
		myDbBean.setMyQuery(myQuery);
	    myDbBean.execute_sql(myQuery);

		// update the balance table
	    myQuery = "";
		myQuery += "UPDATE balance set amount = " + anOrder.getSumTotal();
	    myQuery += " WHERE account_id = " + anOrder.getAccountID();
System.out.println(myQuery);
		myDbBean.setMyQuery(myQuery);
	    myDbBean.execute_sql(myQuery);
   }


/*---------------------------------------------------------
    User will enter customer name and/or accountID, and billing cycle(date range)
    for reconciled.
    ? Date range and multiple customers are allowed.
    ? The result returns transactions (payments and adjustments) between
      bStart and bEnd listed by accountID.

    Call: public int[] getTidByAccountIDDates (int accoundID, String bStart, String bEnd)
    ? User reconciles the account and result will be shown in a separate window.

    retrieve amount and calculate them.
    *******************************************************************
    * Method: void Reconciliation (int accountID, bStart, bEnd)
    * steps:
     *******************************************************************/

   public void Reconciliation (int aID, String bStart, String bEnd) throws Exception
   {
	 	String inv = "INV";
	   	Vector resultV = new Vector();

	  	myQuery = "";
	   	myQuery = "select * from transaction WHERE account_id = " + aID;
	   	myQuery += " AND timestamp >= " + bStart + " AND timestamp <= " + bEnd;
System.out.println(myQuery);

	    myDbBean.setMyQuery(myQuery);
		myDbBean.execute_sql(myQuery);

	    if (myDbBean.loadData()) {

		  	while (myDbBean.getNextRecord())
		  	{
               // retrieve the value of amount.
               // add them up.
               // update the amount in balance table.
		  	}
  	   }
   }
}