
/***************************************************************************************/
// Program Name: CardUtilization.java
//
// 
//
// Date Written: Feb, 2001
//
// Programmer: H. Huang
//
// Purpose:  To lookup calling card utilization data in the database. 
//
//
// Update(s):
/**************************************************************************************/

package portal.report;
import portal.bean.*;

import java.util.*;
import java.sql.*;
import java.io.*;


public class CardUtilization 
{
   private DBApp db_App;
   protected DebugWriter writer;
   protected FilePageWriter fpWrite;

   private String sqlcmd ;

   private String action  ;
   private String ISP_Name  ;
   private String startDate ;
   private String startHH ;
   private String startMI ;
   private String startSS ;
   private String endDate  ;
   private String endHH ;
   private String endMI  ;
   private String endSS  ;
   private String sort ;
   private String tableName ;
   private String fileName ;

   private String paramList ;
   private static int fileCounter=0;

 

  
/********************************************************************
* Method:   CardUtilization()  
*
* Purpose:  Constructor  
*********************************************************************/

  public CardUtilization() {
    action = "";      
    ISP_Name= "";
    startDate = "" ;
    endDate = "" ;
    sort = "" ;
    fileName = "";

    db_App  = new DBApp();
    writer = new DebugWriter();
    fpWrite = new FilePageWriter();

  }
 


/********************************************************************
* Method:  void getReport()  
*
* Purpose:  This method returns the fileName
*           
*********************************************************************/

 public String getReport() { 
    return ( fpWrite.getXmlDirName() +  fileName+".xml" );
 }



/********************************************************************
* Method:  void getReport(String[] sList)
*
* Purpose:  This method returns the file name of a report
*
*********************************************************************/

 public String getReport(String[] sList) {
    doReport(sList);
    return ( fpWrite.getXmlDirName()  + fileName+".xml" );
 }

/********************************************************************
* Method:   void doReport(String[] sList )
*
* Purpose:  Start getting report from here.
*********************************************************************/

   public void doReport(String[] sList )
   {
      setReportParams(sList);
      buildQuery() ;
      executeQuery();
      processData()  ;
      saveFile();

   }


  
/********************************************************************
* Method:   void setReportParams()
*
* Purpose:  Set the report params.
.
*********************************************************************/

 public void setReportParams(String[] sList)  {
    action = sList[0];
     ISP_Name = sList[1];   
     startDate = sList[2];  

     startHH = sList[3]; 
     if( startHH.equals("") ) { startHH = "00" ; }

     startMI = sList[4]; 
     if( startMI.equals("") ) { startMI = "00" ; }

     startSS = sList[5]; 
     if( startSS.equals("") ) { startSS = "00" ; }

     endDate = sList[6]; 
     endHH = sList[7]; 
     if( endHH.equals("") ) { endHH = "00" ; }

     endMI = sList[8];  
     if( endMI.equals("") ) { endMI = "00" ; }

     endSS = sList[9];  
     if( endSS.equals("") ) { endSS = "00" ; }

     sort = sList[10];  
     tableName = sList[11];  
     
     
    paramList = " action =\"" + action + "\" " +
                  "ISP_Name=\"" + ISP_Name + "\" " +
                  "startDate=\"" + startDate + "\" " +
                  "startHH=\"" + startHH + "\" " +
                  "startMI=\"" + startMI + "\" " +
                  "startSS=\"" + startSS  + "\" " +
                  "endDate =\"" + endDate  + "\" " +
                  "endHH=\"" + endHH + "\" " +
                  "endMI=\"" + endMI + "\" " +
                  "endSS=\"" + endSS + "\" " +
                  "sort=\"" + sort + "\" " +
                  "tableName=\"" + tableName + "\" " ;


 }

  
  
/********************************************************************
* Method:   void buildQuery()
*
* Purpose:  This method builds sql query.
*********************************************************************/

  private void buildQuery(){
     String sortOrder = "";
     String space = " ";
     String colon = ":";

     if (sort.equals("CallingCardDescription")){
          sortOrder = "p.description";
     }
     else if (sort.equals("Minutes")) {
        sortOrder = " round(sum(v.duration)/60, 0)"; 
     }
     else {      // (sort.equals("Provider") )
        sortOrder = " round(sum(v.duration)/60, 0)"; 
     }
               
    if( ISP_Name.equals("ALL") ) {
       sqlcmd =  "select i.name,  p.description, round(sum(v.duration)/60, 0)";
        
     }else {
       sqlcmd =  "select  p.description, round(sum(v.duration)/60, 0)";
     }
     
	sqlcmd +=  " from service s,  product p,  voip_cdr v, ispvn i " ;
	
    sqlcmd = sqlcmd + "where upper(i.name) = upper('"+ ISP_Name + "') ";
    
    if( ! startDate.equals("") ){
        sqlcmd = sqlcmd + "and v.timestamp  >= to_date('"
                   + startDate + space+ startHH + colon + startMI + colon + startSS 
                   + "','mm/dd/yyyy HH24:MI:SS') ";
     }
     if( ! endDate.equals("") ){
        sqlcmd = sqlcmd +"   and v.timestamp   <= to_date('"
                 + endDate + space+ endHH + colon + endMI+ colon + endSS 
                 + "','mm/dd/yyyy HH24:MI:SS') ";
      
    }
    
     sqlcmd +=  " and i.id = p.ispvn_id  ";                                                                 
     sqlcmd +=  " and v.service_id = s.id ";
     sqlcmd +=  " and s.product_id = p.id ";

     sqlcmd +=  " group by   p.description ";
     sqlcmd += " order by " + sortOrder;
   
     db_App.setMyQuery( sqlcmd );
   //  writer.writeDebug(sqlcmd);
     System.out.println(sqlcmd);

  }



/********************************************************************
* Method:  executeQuery() 
*
* Purpose:  execute sql query
*********************************************************************/

private void executeQuery(){
     
      try {
         db_App.makeConnection(); 
         db_App.execute_sql(sqlcmd) ; 

      }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
      }
}


/********************************************************************
* Method:  processData() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

  private void processData()  {
    int  noColumn = 0;
  
    try {

     noColumn = db_App.getNoColumn();

     String[] headLine = new String[noColumn];
     if( ISP_Name.equals("ALL") ){
           headLine[0] = "Provider"  ; 
           headLine[1] = "CallingCardDescription"; 
           headLine[2] = "Minutes" ; 
   
     }else {          
           headLine[0] = "CallingCardDescription"; 
           headLine[1] = "Minutes" ; 
           
     }

     fpWrite.putResultInVector(db_App,headLine); 

    }  catch (Exception ex) {   
            System.out.println("Error in processData:    "+ex);
    }
}


/********************************************************************
* Method:  void saveFile()  
*
* Purpose:  This method returns a page to the user on the web
*           or print this page to a file.
*********************************************************************/


 private void saveFile()  { 
   try{
     fpWrite.setTitle("cardUtilization");
     fpWrite.setParamList(paramList);

     fpWrite.setProvider(ISP_Name);

     fileCounter++;
     if(fileCounter > 100) fileCounter = 1;
     fileName = "cardUtilization_" + fileCounter ;

     fpWrite.setFileName(fileName);
   //  fpWrite.writeFileV(fileName+".csv") ;
     fpWrite.writeXMLFileV(fileName+".xml") ;

   
   }  catch (Exception ex) {   
            System.out.println("Error in saveFile: "+ex);
   }

 }


}