
// Denied.java
/***************************************************************************************/
// Program Name: Denied.java
//
// 
//
// Date Written: January, 2001
//
// Programmer: H. Huang
//
// Purpose:  To lookup "Number of Customers Denied Service" data in the database.
//
//
// Update(s):
/**************************************************************************************/


package portal.report;
import portal.bean.*;

import java.sql.*;
import java.io.*;


public class Denied 
{
   private DBApp db_App;
   protected DebugWriter writer;
   protected FilePageWriter fpWrite;

   private String sqlcmd ;
   private String action  ;

   private String ISP_Name ="" ;
   private String startDate ;
   private String endDate  ;
   private String sort ;
   private String tableName ;
   private String fileName ;

   private String paramList ;
   private static int fileCounter=0;


  
/********************************************************************
* Method:   Denied()  
*
* Purpose:  Constructor
*********************************************************************/

  public Denied() {
    action = "";      
    ISP_Name= "";
    startDate = "" ;
    endDate = "" ;
    sort = "" ;
    fileName = "";

    fpWrite = new FilePageWriter();
    db_App  = new DBApp();
    writer = new DebugWriter();

  }



/********************************************************************
* Method:  void getReport(String[] sList)
*
* Purpose:  This method returns the file name of a report
*
*********************************************************************/

 public String getReport(String[] sList) {
    doReport(sList);
    return ( fpWrite.getXmlDirName()  + fileName+".xml" );
 }



/********************************************************************
* Method:   void doReport(String[] sList )
*
* Purpose:  Start getting report from here.
*********************************************************************/

   public void doReport(String[] sList )
   {
      setReportParams(sList);
      buildQuery() ;
      executeQuery();
      processData()  ;
      saveFile();

   }

  
    
/********************************************************************
* Method:   void setReportParams()
*
* Purpose:  Set the report parameters.
*
*	  action = sList[0];
*     ISP_Name = sList[1];   
*     startDate = sList[2];  
*     endDate = sList[3];     
*     sort = sList[4];  
*********************************************************************/

 public void setReportParams(String[] sList)  {	
 	action = sList[0];
      ISP_Name = sList[1];   
      startDate = sList[2];  
      endDate = sList[3];   
      sort = sList[4];  
      tableName = sList[5];   

      paramList = " action =\"" + action + "\" " +
                  "ISP_Name=\"" + ISP_Name + "\" " +
                  "startDate=\"" + startDate + "\" " +               
                  "endDate =\"" + endDate  + "\" " +           
                  "sort=\"" + sort + "\" " +
                  "tableName=\"" + tableName + "\" " ;
              
 }


/********************************************************************
* Method:   void buildQuery(  )
*
* Purpose:  This method builds sql query.
*********************************************************************/

  private void buildQuery( ){
     String sortOrder = "";

     if (sort.equals("Date")){
        sortOrder = "to_char(v.timestamp,'mm/dd/yyyy') ";
     }
     else if (sort.equals("Account_id")){
        sortOrder = "v.account_id";
     }
     else if (sort.equals("Disconnect Cause")){
        sortOrder = "v.disconnect_cause";
     }
     else if (sort.equals("Connection_id")) {
        sortOrder = "v.connection_id "; 
     }
     else if (sort.equals("Duration")) {
        sortOrder = "round(v.duration/60,0) "; 
     }
     else if (sort.equals("Orig_pop_id")) {
        sortOrder = "v.originating_pop_id "; 
     }
     else {      // (sort.equals("Provider") )
        sortOrder = "i.name"; 
     }

     if( ISP_Name.equals("ALL") ) {
        sqlcmd = " select i.name,v.account_id, v.disconnect_cause, v.connection_id, ";
     }else {
        sqlcmd = " select v.account_id, v.disconnect_cause, v.connection_id, ";
     }

     sqlcmd +=  " round(v.duration/60,0) , " ;
     sqlcmd +=  " v.originating_pop_id, to_char(v.timestamp,'mm/dd/yyyy') ";  //, v.terminating_pop_id "  ; 
     sqlcmd = sqlcmd + " from " + tableName + " v ,  ispvn i ";                 
   //  sqlcmd += " from   voip_cdr_failed v ,  ispvn i ";
     sqlcmd += " where  i.id = v.ispvn_id ";

     if( ! ISP_Name.equals("ALL") ){
         sqlcmd = sqlcmd  + " and i.name = '" + ISP_Name  +"'";
     }
     if( ! startDate.equals("") ){
         sqlcmd = sqlcmd + " and v.timestamp >= TO_DATE('" + startDate + "', 'MM/DD/YYYY') " ;
     }
     if( ! endDate.equals("") ){
         //  endDate = ADD_MONTHS(endDate +1);
          sqlcmd +=  " and v.timestamp < TO_DATE('" + endDate + "', 'MM/DD/YYYY')+1" ;
     }

     sqlcmd += " order by " + sortOrder;

     db_App.setMyQuery( sqlcmd );
   //  writer.writeDebug(sqlcmd);
     System.out.println(sqlcmd);

  }



/********************************************************************
* Method:  executeQuery() 
*
* Purpose:  execute sql query
*********************************************************************/

private  void executeQuery(){
     
      try {
         db_App.makeConnection(); 
         db_App.execute_sql(sqlcmd) ; 

      }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
      }
}



/********************************************************************
* Method:  processData() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

  private  void processData()  {
    int  noColumn = 0;

    try {

     noColumn = db_App.getNoColumn();

     String[] headLine = new String[noColumn];
     if( ISP_Name.equals("ALL") ){
           headLine[0] = "Provider"  ; 
           headLine[1] = "Account_id"  ; 
           headLine[2] = "Disconnect Cause"; 
           headLine[3] = "Connection_id" ; 
           headLine[4] = "Num of Minutes" ; 
           headLine[5] = "Orig_pop_id" ; 
           headLine[6] = "Date " ; 
      
     }else {          
           headLine[0] = "Account_id"  ; 
           headLine[1] = "Disconnect Cause"; 
           headLine[2] = "Connection_id" ; 
           headLine[3] = "Num of Minutes" ; 
           headLine[4] = "Orig_pop_id" ; 
           headLine[5] = "Date " ; 
 
     }

     fpWrite.putResultInVector(db_App,headLine); 

    }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
    }
}


/********************************************************************
* Method:  void saveFile()  
*
* Purpose:  This method writes the query result 
*           to a file.
*********************************************************************/


 private  void saveFile()  { 
   try{
     fpWrite.setTitle("Denied Service");
     fpWrite.setParamList(paramList);

     fpWrite.setProvider(ISP_Name);

     fileCounter++;
     if(fileCounter > 100) fileCounter = 1;
     fileName = "denied_" + fileCounter ;

     fpWrite.setFileName(fileName);
  //   fpWrite.writeFileV(fileName+".csv") ;
     fpWrite.writeXMLFileV(fileName+".xml") ;

   
   }  catch (Exception ex) {   
            System.out.println("Error in saveFile: "+ex);
   }

 }





}