
// DeniedNum.java
/***************************************************************************************/
// Program Name: DeniedNum .java
//
// 
//
// Date Written: March, 2001
//
// Programmer: H. Huang
//
// Purpose:  To lookup "Number of Customers Denied Service" data in the database.
//
//
// Update(s):
/**************************************************************************************/

package portal.report;
import portal.bean.*;

import java.text.*;
import java.util.*;
import java.sql.*;
import java.io.*;


public class DeniedNum 
{
   private DBApp db_App;
   protected DebugWriter writer;
   protected FilePageWriter fpWrite;

   private String sqlcmd ;
   private String action  ;

   private String ISP_Name ="" ;
   private String startDate ;
   private String endDate  ;

   private String sort ;
   private String tableName ;
   private String fileName ;

   private  Vector resultSetV ;

   private String paramList ;
   private static int fileCounter=0;

 
  
/********************************************************************
* Method:   DeniedNum()  
*
* Purpose:  Constructor
*********************************************************************/

  public DeniedNum() {
    action = "";      
    ISP_Name= "";
    startDate = "" ;
    endDate = "" ;
    sort = "" ;
    fileName = "";

    fpWrite = new FilePageWriter();

    db_App  = new DBApp();
    writer = new DebugWriter();

  }

 

/********************************************************************
* Method:  void getReport(String[] sList)
*
* Purpose:  This method returns the file name of a report
*
*********************************************************************/

 public String getReport(String[] sList) {
    doReport(sList);
    return ( fpWrite.getXmlDirName()  + fileName+".xml" );
 }



/********************************************************************
* Method:   void doReport(String[] sList )
*
* Purpose:  Start getting report from here.
*********************************************************************/

   public void doReport(String[] sList )
   {
      setReportParams(sList);
      buildQuery() ;
      executeQuery();
      processData()  ;
      saveFile();

   }

  
    
/********************************************************************
* Method:   void setReportParams()
*
* Purpose:  Set the report parameters.
*
*	  action = sList[0];
*     ISP_Name = sList[1];   
*     startDate = sList[2];  
*     endDate = sList[3];     
*     sort = sList[4];  
*********************************************************************/

 public void setReportParams(String[] sList)  {	
 	action = sList[0];
      ISP_Name = sList[1];   
      startDate = sList[2];  
      endDate = sList[3];   
      sort = sList[4];  
      tableName = sList[5];   

      paramList = " action =\"" + action + "\" " +
                  "ISP_Name=\"" + ISP_Name + "\" " +
                  "startDate=\"" + startDate + "\" " +               
                  "endDate =\"" + endDate  + "\" " +           
                  "sort=\"" + sort + "\" " +
                  "tableName=\"" + tableName + "\" " ;
              
 }



/********************************************************************
* Method:   void buildQuery( )
*
* Purpose:  This method builds sql query.
*********************************************************************/

  public void buildQuery( ){
     String sortOrder = "";

     if (sort.equals("Disconnect Cause")){
        sortOrder = "v.disconnect_cause";
     }
     else if (sort.equals("Number of Times")) {
        sortOrder = "count(*) "; 
     }
     else {      // (sort.equals("Provider") )
        sortOrder = "i.name"; 
     }

     if( ISP_Name.equals("ALL") ) {
        sqlcmd = " select i.name, v.disconnect_cause, count(*) "  ;  
     }else {
        sqlcmd = " select v.disconnect_cause, count(*) "  ;  
     }

     sqlcmd = sqlcmd + " from " + tableName + " v ,  ispvn i ";                            
     sqlcmd += " where  i.id = v.ispvn_id ";

     if( ! ISP_Name.equals("ALL") ){
         sqlcmd = sqlcmd  + " and i.name = '" + ISP_Name  +"'";
     }
     if( ! startDate.equals("") ){
         sqlcmd = sqlcmd + " and v.timestamp >= TO_DATE('" + startDate + "', 'MM/DD/YYYY') " ;
     }
     if( ! endDate.equals("") ){
         //  endDate = ADD_MONTHS(endDate +1);
          sqlcmd +=  " and v.timestamp < TO_DATE('" + endDate + "', 'MM/DD/YYYY')+1" ;
     }

     sqlcmd += " group by i.name, v.disconnect_cause " ;
     sqlcmd += " order by " + sortOrder;

     db_App.setMyQuery( sqlcmd );
 //    writer.writeDebug(sqlcmd);
 

     System.out.println(sqlcmd);

  }



/********************************************************************
* Method:  executeQuery() 
*
* Purpose:  execute sql query
*********************************************************************/

public void executeQuery(){
     
      try {
         db_App.makeConnection(); 
         db_App.execute_sql(sqlcmd) ; 

      }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
      }
}



/********************************************************************
* Method:  processData() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

  private void processData()  {
    int  noColumn = 0;

    try {

     noColumn = db_App.getNoColumn()+1;

     String[] headLine = new String[noColumn];
     if( ISP_Name.equals("ALL") ){
           headLine[0] = "Provider"  ; 
           headLine[1] = "Disconnect Cause"  ; 
           headLine[2] = "Num of Times"; 
           headLine[3] = "Times(%)" ; 
      
     }else {          
           headLine[0] = "Disconnect Cause"  ; 
           headLine[1] = "Num of Times"; 
           headLine[2] = "Times(%)" ; 

     }

     putResultInVector(db_App,headLine); 

    }  catch (Exception ex) {   
            System.out.println("Error in processData:    "+ex);
    }
}



/********************************************************************
* Method:  putResultInVector() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

public void putResultInVector(DBApp db_App, String[] fieldList)  throws Exception {

     int  no = 0;
     int totalTimes = 0;
   
     resultSetV = new Vector();

     resultSetV.addElement(fieldList); 

     if (db_App.loadData()){
        while (db_App.getNextRecord()) {
          no++;
     
          String disconCause = new String(db_App.getColumn(1));
          int times = db_App.getColumnInt(2);
          String timesP = new String("");

          String[] recordS = {disconCause, Integer.toString(times ),timesP  }  ;
          resultSetV.addElement(recordS);
          totalTimes += times ; 
        } // end of while
        fpWrite.setRecordNum(no);
      }  //end of if
      db_App.takeDown();

      String[] recordS = {"Total",  Integer.toString(totalTimes), ""  }  ;
      resultSetV.addElement(recordS);

     /************************************************************
        Compute the percentage value of times in every row,
        and put it in the vector.
        Ignore the first row which is the header and the last row
        which is the total. 
     *************************************************************/

     for (int i=1; i < resultSetV.size()-1; i++ ) {           
           String[] column = (String[]) ((resultSetV.elementAt(i)) ) ;
           String times   = column[1];    
           int t = Integer.parseInt(times);

           
           /*  Format output:   setMaximumFractionDigits = 2  */
           
           NumberFormat nf = NumberFormat.getNumberInstance();
           nf.setMaximumFractionDigits(2);
 
           double timePct = (double) t*100 / (double)totalTimes  ;
           String  timePctS = nf.format(timePct   );
           column[2] = timePctS ;        
      }

      fpWrite.setResultSetV(resultSetV);


}


/********************************************************************
* Method:  void saveFile()  
*
* Purpose:  This method writes the query result 
*           to a file.
*********************************************************************/


 private void saveFile()  { 
   try{
     fpWrite.setTitle("Denied Service");
     fpWrite.setParamList(paramList);

     fpWrite.setProvider(ISP_Name);

     fileCounter++;
     if(fileCounter > 100) fileCounter = 1;
     fileName = "denied_" + fileCounter ;

     fpWrite.setFileName(fileName);
   //  fpWrite.writeFileV(fileName+".csv") ;
     fpWrite.writeXMLFileV(fileName+".xml") ;

   
   }  catch (Exception ex) {   
            System.out.println("Error in saveFile: "+ex);
   }

 }


}