
// RevenueDestination.java
/***************************************************************************************/
// Program Name: RevenueDestination.java
//
// Copyright (c)2001 iBasis.    All rights reserved.
// Author: H. Huang
//
// Date Written: Januay, 2001
//
//
// Purpose:  To retrieves information on revenue (Destination summary)  using input
//           from the user via html.
//
//
// Update(s):  1) H. Huang,  May,2001  
//                Add "Export" option
/**************************************************************************************/


package portal.report;
import portal.bean.*;

import java.sql.*;
import java.io.*;
import java.util.Vector;


public class RevenueDestination 
{
   private DBApp db_App;
   protected DebugWriter writer;
   protected FilePageWriter fpWrite;

   private String sqlcmd ;
   private String action  ;

   private String ISP_Name ="" ;
   private String startDate ;
   private String endDate  ;

   private String sort ;
   private String fileName ;

   private String paramList ;
   private static int fileCounter;
   private Vector resultVector ;
  
 
  
/********************************************************************
* Method:   RevenueDestination()  
*
* Purpose:  Constructor
*********************************************************************/

  public RevenueDestination() {
    action = "";      
    ISP_Name= "";
    startDate = "" ;
    endDate = "" ;
    sort = "" ;
    fileName = "";
       
    paramList = "";  
    fileCounter = 0;
    resultVector = null;

    db_App  = new DBApp();
    writer = new DebugWriter();
    fpWrite = new FilePageWriter();
    
  }



  /********************************************************************
* Method:  void cleanup()  
*
* Purpose:  This method does cleanup. 
*           Release some resource after susing it.
*           
*********************************************************************/
  private void cleanup() { 
    resultVector = null;    // After using it, clean it.
  }

  
/********************************************************************
* Method:  void getReport()  
*
* Purpose:  This method returns the fileName
*           
*********************************************************************/

 public String getReport() { 
    return ( fpWrite.getXmlDirName()  +  fileName+".xml" );
 }

  

/********************************************************************
* Method:  void getReport(String[] sList)
*
* Purpose:  This method returns the file name of a report
*
*********************************************************************/

 public String getReport(String[] sList) {
    doReport(sList);
    return ( fpWrite.getXmlDirName()  + fileName+".xml" );
 }



/********************************************************************
* Method:   void doReport(String[] sList )
*
* Purpose:  Start getting report from here.
*********************************************************************/

   public void doReport(String[] sList )
   {
      setReportParams(sList);
      buildQuery() ;
      executeQuery();
      processData()  ;
      saveFile();
      cleanup();
   }


   
/********************************************************************
* Method:   void setReportParams()
*
* Purpose:  Set the report parameters.
*
*	  action = sList[0];
*     ISP_Name = sList[1];   
*     startDate = sList[2];  
*     endDate = sList[3];     
*     sort = sList[4];  
*********************************************************************/

 public void setReportParams(String[] sList)  {	
 	  action = sList[0];
      ISP_Name = sList[1];   
      startDate = sList[2];  
      endDate = sList[3];     
      sort = sList[4];  
   
      setParamList() ;
              
 }

/********************************************************************
* Method:   void setParamList()
*
* Purpose:  Set the report params header.
*    
*    
*********************************************************************/

 public void setParamList()  { 
    
 	  paramList = " action =\"" + action + "\" " +
                  "ISP_Name=\"" + ISP_Name + "\" " +
                  "startDate=\"" + startDate + "\" " +               
                  "endDate =\"" + endDate  + "\" " +           
                  "sort=\"" + sort + "\" " ;  
 }



/********************************************************************
* Method:   void buildQuery(  )
*
* Purpose:  This method builds sql query.
*********************************************************************/

  private void buildQuery(  ){
     String sortOrder = "";

     if (sort.equals("Calls")){
        sortOrder = "count(*)";
     }
     else if (sort.equals("Destination")) {
        sortOrder = "t.name"; 
     }
     else if (sort.equals("Minutes")) {
        sortOrder = "round(sum(v.duration/60),0)"; 
     }
     else if (sort.equals("Revenue")) {
        sortOrder = "sum(v.amount )  DESC  "; 
     }
     else if (sort.equals("Provider")){
        sortOrder = "i.name"; 
     }
     else{   sortOrder = "" ;    }

     if( ISP_Name.equals("ALL") ) {
     
     	sqlcmd =  " select i.name, t.name, count(*) , round(sum(v.duration/60),0) , round(sum(v.amount ),2) ";
    
    }else {
      
       sqlcmd =  " select t.name, count(*) , round(sum(v.duration/60),0) , round(sum(v.amount ),2) ";
       
     }

    //sqlcmd =  " select i.name, t.name, count(*) , round(sum(v.duration/60),0) , round(sum(v.amount ),2) ";
     sqlcmd +=   " from voip_cdr v, ispvn i, product p, service s, tele_numplan t " 
            + " where i.id             = v.ispvn_id "
            + " and t.id          = v.dst_tele_numplan_id "
            + " and v.service_id    	= s.id "
            + " and s.product_id = p.id  "
            + " and v.ispvn_id       = p.ispvn_id " 
            + " and v.duration         > 0  and v.disconnect_cause in (16, 31) " ;

     if( ! ISP_Name.equals("ALL") ){
         sqlcmd = sqlcmd  + " and i.name = '" + ISP_Name  +"'";
     }
     if( ! startDate.equals("") ){
         sqlcmd = sqlcmd + " and v.timestamp >= TO_DATE('" + startDate + "', 'MM/DD/YYYY') " ;
     }
     if( ! endDate.equals("") ){      
          sqlcmd +=  " and v.timestamp < TO_DATE('" + endDate + "', 'MM/DD/YYYY')+1" ;
     }

     sqlcmd = sqlcmd  + " group by i.name, t.name ";
     if( !(sort.equals("")) ){
        sqlcmd +=  " order by " + sortOrder; 
     }
  
     db_App.setMyQuery( sqlcmd );
   //  writer.writeDebug(sqlcmd);
     System.out.println(sqlcmd);

  }



/********************************************************************
* Method:  executeQuery() 
*
* Purpose:  execute sql query
*********************************************************************/

private void executeQuery(){    
      try {
         db_App.makeConnection(); 
         db_App.execute_sql(sqlcmd) ; 

      }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
      }
}




/********************************************************************
* Method:  processData() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

  private void processData()  {
    int  noColumn = 0;
  
    try {

     noColumn = db_App.getNoColumn();

     String[] headLine = new String[noColumn];
     if( ISP_Name.equals("ALL") ){
     	
           headLine[0] = "Provider"  ; 
           headLine[1] = "Destination"; 
           headLine[2] = "Num of Calls"; 
           headLine[3] = "Num of Minutes" ; 
           headLine[4] = "Revenue" ; 
      
     }else {  
     	   headLine[0] = "Destination";         
           headLine[1] = "Num of Calls"; 
           headLine[2] = "Num of Minutes" ; 
           headLine[3] = "Revenue" ; 
 
     } 
      resultVector = fpWrite.getResultVector(db_App,headLine);

    }  catch (Exception ex) {   
            System.out.println("Error processData:    "+ex);
    }
}


/********************************************************************
* Method:  void saveFile()  
*
* Purpose:  This method writes the query result 
*           to a file.
*********************************************************************/


 private void saveFile()  { 
   try{
     fpWrite.setTitle("revenue (Destination Summary) ");
     fpWrite.setParamList(paramList);

     fpWrite.setProvider(ISP_Name);

     fileCounter++;
     if(fileCounter > 100) fileCounter = 1;
     fileName = "revenueDestination_" + fileCounter ;

 //    fpWrite.writeFileV(fileName+".csv", resultVector) ;
     fpWrite.writeXMLFileV(fileName+".xml", resultVector) ;
       
   }  catch (Exception ex) {   
            System.out.println("Error saveFile: "+ex);
   }

 }


}