
//  TrafficByHour.java
/***************************************************************************************/
// Program Name: TrafficByHour.java
// Copyright (c)2001 iBasis.    All rights reserved.
// Author: H. Huang
// 
//
// Date Written: Jan. 2001
//
// Programmer: H. Huang
//
// Purpose:  To lookup rafficByDay data in the database using input
//           from the user via html.
//
//
// Update(s):  1. H. Huang,  July,,2001  
//                Add "Export" option
//
//
// Note:   action = "View" or "Export"
//         ParamList =
//                     {"action",
//                      "ISP_Name",
//                      "startDate",    
//                      "startHH",       
//                      "endDate",    
//                      "endHH",   
//                      "sort"}
/**************************************************************************************/


package portal.report;
import portal.bean.*;

import java.sql.*;
import java.io.*;
import java.util.Vector;


public class TrafficByHour
{
   private DBApp db_App;
   protected DebugWriter writer; 
   protected FilePageWriter fpWrite;
   private String sqlcmd ;
   private String action  ;
   private String ISP_Name ="" ;
   private String startDate ;
   private String startHH ;
   private String endDate  ;
   private String endHH ;
   private String sort ;
   private String fileName ;
   private String paramList ;   
   private static int fileCounter;

   private Vector resultVector ;
 
  
/********************************************************************
* Method:   TrafficByDay()  
*
* Purpose:  Constructor
*********************************************************************/

  public TrafficByHour() {
    action = "";      
    ISP_Name= "";
    startDate = "" ;   
    startHH = ""; 
    endDate = "" ;
    endHH = "";    
    sort = "" ;
    fileName = "";
    paramList = "";  
    fileCounter = 0;
    resultVector = null;
    fpWrite = new FilePageWriter();
    
    db_App  = new DBApp();
    writer = new DebugWriter();
  }


/********************************************************************
* Method:  void cleanup()  
*
* Purpose:  This method does cleanup. 
*           Release some resource after susing it.
*           
*********************************************************************/
  private void cleanup() { 
	    resultVector = null;    // After using it, clean it.
	}

/********************************************************************
* Method:  void getReport()  
*
* Purpose:  This method returns the fileName
*           
*********************************************************************/

 public String getReport() { 
    return ( fpWrite.getXmlDirName()  +  fileName+".xml" );
 }

  
/********************************************************************
* Method:  void getReport(String[] sList)
*
* Purpose:  This method returns the file name of a report
*
*********************************************************************/

public String getReport(String[] sList) {
   String fileL = "";
    doReport(sList);
    if(action.equals("View")){
       fileL = fpWrite.getXmlDirName()  + fileName+".xml" ;
       System.out.println("fileName = " + fileL );
     }else {        	  //(action.equals("Export") )
       fileL = fileName+".csv" ;
       System.out.println("fileName = " + fileL );
    }
    return fileL;
 }


/********************************************************************
* Method:   void doReport(String[] sList )
*
* Purpose:  Start getting report from here.
*********************************************************************/

   public void doReport(String[] sList )
   {
      setReportParams(sList);
      buildQuery() ;
      executeQuery();
      processData()  ;
      saveFile();
      cleanup() ; 
   }
  
    
/********************************************************************
* Method:   void setReportParams()
*
* Purpose:  Set the report params.
*
*	  action = sList[0];
*     ISP_Name = sList[1];   
*     startDate = sList[2];  
*     endDate = sList[3];     
*     sort = sList[4];  
*********************************************************************/

 public void setReportParams(String[] sList)  {
  	 action = sList[0];
     ISP_Name = sList[1];   
     startDate = sList[2];        
     startHH = sList[3]; 
     if( startHH.equals("") ) { startHH = "00" ; }    
     endDate = sList[4];       
     endHH = sList[5];    
     if( endHH.equals("") ) { endHH = "00" ; }        
     sort = sList[6];  
     setParamList() ;
 }

/********************************************************************
* Method:   void setParamList()
*
* Purpose:  Set the report params header.
*    
*    
*********************************************************************/

 public void setParamList()  { 
 	    paramList = " action =\"" + action + "\" " +
                  "ISP_Name=\"" + ISP_Name + "\" " +
                  "startDate=\"" + startDate + "\" " +   
                  "startHH=\"" + startHH + "\" " +             
                  "endDate =\"" + endDate  + "\" " +  
                  "endHH=\"" + endHH + "\" " +          
                  "sort=\"" + sort + "\" " ;   
 }

  
/********************************************************************
* Method:   void buildQuery(HttpServletRequest request)
*
* Purpose:  This method builds sql query.
*********************************************************************/

  private void buildQuery(){
     String sortOrder = " to_char(v.timestamp,'mm/dd/yy hh24') ";
     String space = " ";
     String colon = ":";

     if (sort.equals("Date")){
        sortOrder = " to_char(v.timestamp,'mm/dd/yy hh24')";
     }
     else if (sort.equals("Traffic")) {
        sortOrder = "round(sum(v.duration/60),0)"; 
     }
     else {    
       // sortOrder = "i.name"; 
     }

     if( ISP_Name.equals("ALL") ) {
         sqlcmd =  "select i.name , to_char(v.timestamp,'mm/dd/yy hh24'), round(sum(v.duration/60),0) " ;
      
     }else {
	     sqlcmd =  "select  to_char(v.timestamp,'mm/dd/yy hh24'), round(sum(v.duration/60),0) " ;
    
     }
  
     sqlcmd = sqlcmd  + " from voip_cdr v, ispvn i, product p, service s  " ;
     sqlcmd = sqlcmd  +"  where i.id  = v.ispvn_id    and v.service_id  = s.id    and s.product_id = p.id  ";
     sqlcmd = sqlcmd  + " and v.ispvn_id   = p.ispvn_id  and v.duration > 0  and v.disconnect_cause in (16,31)  ";

     if( ! ISP_Name.equals("ALL") ){
          sqlcmd = sqlcmd  + " and i.name = '" + ISP_Name  +"'";
     }
     if( ! startDate.equals("") ){
         // if (startDate.length() == 10 ) { startDate = startDate + " 00" ; }
          sqlcmd = sqlcmd + " and v.timestamp >= TO_DATE('" + startDate + space+ startHH + "', 'mm/dd/yyyy hh24') " ;
     }
     if( ! endDate.equals("") ){
         // if (endDate.length() == 10 ) { endDate = endDate + " 00" ; }
		sqlcmd +=  " and v.timestamp < TO_DATE('" + endDate  + space+ endHH +"', 'MM/DD/YYYY hh24')" ;
         // sqlcmd +=  " and v.timestamp < TO_DATE('" + endDate + "', 'MM/DD/YYYY hh24')+1" ;
     }

     sqlcmd = sqlcmd  + "  group by i.name, to_char(v.timestamp,'mm/dd/yy hh24') " ;
     sqlcmd += " order by " + sortOrder;
   
     db_App.setMyQuery( sqlcmd );
  //   writer.writeDebug(sqlcmd);
     System.out.println(sqlcmd);

  }



/********************************************************************
* Method:  executeQuery() 
*
* Purpose:  execute sql query
*********************************************************************/

private void executeQuery(){
     
      try {
         db_App.makeConnection(); 
         db_App.execute_sql(sqlcmd) ; 

      }  catch (Exception ex) {   
            System.out.println("Error Connection:    "+ex);
      }
}



/********************************************************************
* Method:  processData() 
*
* Purpose:  Put the query result into a vector.
*********************************************************************/

  private void processData()  {
    int  noColumn = 0;
  
    try {

     noColumn = db_App.getNoColumn();

     String[] headLine = new String[noColumn];
     if( ISP_Name.equals("ALL") ){
           headLine[0] = "Provider"  ; 
           headLine[1] = "DATE" ; 
           headLine[2] = "TRAFFIC";      
     }else {          
           headLine[0] = "DATE" ; 
           headLine[1] = "TRAFFIC";  
     }

     resultVector = fpWrite.getResultVector(db_App,headLine);

    }  catch (Exception ex) {   
            System.out.println("Error in processData:    "+ex);
    }
}


/********************************************************************
* Method:  void saveFile()  
*
* Purpose:  This method writes the query result 
*           to a file.
*********************************************************************/

 private void saveFile()  { 
   try{
     fpWrite.setTitle("trafficByHour");
     fpWrite.setParamList(paramList);
     fpWrite.setProvider(ISP_Name);

     fileCounter++;
     if(fileCounter > 100) fileCounter = 1;
     fileName = "trafficByHour_" + fileCounter ;
     fpWrite.setFileName(fileName);
     
     if(action.equals("View")){     
       fpWrite.writeXMLFileV(fileName+".xml", resultVector) ;
     }else{   // if (action.equals("Export") )   {        	  
       fpWrite.writeFileV(fileName+".csv", resultVector) ;
     }

     // fpWrite.writeFileV(fileName+".csv", resultVector) ;
     //fpWrite.writeXMLFileV(fileName+".xml", resultVector) ;   
   }  catch (Exception ex) {   
            System.out.println("Error in saveFile: "+ex);
   }
 }


}