/*
 * @(#)StreamEncryptor.java	1.0
 * Copyright (c) iBasis Inc.
 *
 */
package portal.servletbeans;

import java.io.*;
import java.net.*;

/**
 * This is a simple Stream Encryptor which uses a combination of
 * random and mathematical functions to generate a sequence of
 * bytes (usually 64) which are added to the actual data to
 * achieve pseudorandom encryption
 */
public class EncryptedInputStream extends InputStream
{

	private static final int PACKET_LENGTH=64;
	private byte[] RandomPacket     = new byte[PACKET_LENGTH];
	private byte[] PacketScrambler  = new byte[PACKET_LENGTH];
	private int xStartPos = 23;
	private int xEndPos   = 41;

	private InputStream srcStream = null;
	private boolean isUninitialized = true;
	private int cursorPos = 0;


	public EncryptedInputStream()
	{
	}

	public void setInputStream(InputStream is)
	{
		srcStream = is;
		isUninitialized = true;
	}


	private void initStream() throws IOException
	{
		//read starting position byte
		xStartPos = srcStream.read();
		//read start scrambler bytes upto xStartPos
		srcStream.read(PacketScrambler, 0, xStartPos);
		//read ending position byte
		xEndPos = srcStream.read();
		//read end scrambler bytes upto xEndPos
		srcStream.read(PacketScrambler, 0, xEndPos);
		//read Encryption Key
		srcStream.read(RandomPacket, 0, PACKET_LENGTH);

		//set flag to indicate that stream has been initialized
		isUninitialized = false;
		cursorPos = 0;

//System.out.println("\nEncrypted Input: "+xStartPos+"  "+xEndPos);
//System.out.println("Random Sequence: ");
//for(int i=0; i < 64; i++) System.out.print((int)(RandomPacket[i])+"  ");
//System.out.println("");
	}


	//Implementation of InputStream
	public int read() throws IOException
	{
		int iValue;

		if(isUninitialized) initStream();

		if(cursorPos < PACKET_LENGTH)
		{
			iValue = srcStream.read() - RandomPacket[cursorPos];
			cursorPos++;
		}
		else
		{
			iValue = srcStream.read() - RandomPacket[0];
			cursorPos = 1;
		}

		return iValue;
	}

	public int read(byte b[]) throws IOException
	{
		int iValue;

		if(isUninitialized) initStream();

		iValue = srcStream.read(b);
//if(iValue > 0)
//{
//System.out.print("Encrypted Input Data: "+iValue+" :\n");
//for(int i=0; i < iValue; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");
//}
		for(int i=0; i < iValue; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[i] -= RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[i] -= RandomPacket[0];
			  cursorPos = 1;
			}
		}

//if(iValue > 0)
//{
//System.out.print("Original Input Data: "+iValue+" :\n");
//for(int i=0; i < iValue; i++) System.out.print((int)(b[i])+"  ");
//System.out.println("");
//}
		return iValue;
	}

	public int read(byte b[], int off, int len) throws IOException
	{
		int iValue;

		if(isUninitialized) initStream();

		iValue = srcStream.read(b, off, len);

		for(int i=0; i < len; i++)
		{
			if(cursorPos < PACKET_LENGTH)
			{
			  b[off+i] -= RandomPacket[cursorPos];
			  cursorPos++;
			}
			else
			{
			  b[off+i] -= RandomPacket[0];
			  cursorPos = 1;
			}
		}

		return iValue;
	}

	public void close() throws IOException
	{
		isUninitialized = true;
		srcStream.close();
	}


	//For Testing purposes only
	/*public static void main(String[] args)
	{
		EncryptedInputStream ti = new EncryptedInputStream();
		try {
		ServerSocket ss = new ServerSocket(8081);

		byte[] data = new byte[80];

		Socket ts = ss.accept();

		long st = System.currentTimeMillis();

		ti.setInputStream(ts.getInputStream());
		int bytesRead = ti.read(data);
		System.out.println("\nData Sequence: "+bytesRead);
		for(int i=0; i < bytesRead; i++)
		{
			System.out.print((char)data[i]);
		}
		System.out.println("\nTimeTaken = "+(System.currentTimeMillis()-st));

		ts.close();
		ss.close();
		} catch (Exception e) { System.out.println("Error: "+e.toString());  }
	}
	*/
}
