package portal.servletbeans;

import java.util.Vector;
import java.io.*;
import java.text.NumberFormat;
//import java.String;

import portal.bean.*;
import portal.invoice.*;
import portal.base.*;
import portal.report.*;
import Node;
import Document;
import Element;
import NodeList;


  	/**
  		* This Bean enables the communication between BillingPortal Servlet
  		* and Invoice Bean. This receives the xml request from the servlet
  		* and calls the respective Invoice bean methods and sends result
  		* back as an xml document
	 */


public class invoiceRequestHandler {

	NumberFormat amtF = NumberFormat.getInstance();

		/**
		 * This document calls the Invoice Bean's createHotBill() to Create the hotBill.
		 * @return   void. The outDoc  xml document will have result after calling the
		 *           underlying Invoice Bean method
		 * @param    inDoc an xml document which describes Accounts for creating Hotbill.
		 * @param    outDoc an empty xml document that will be passed-in and will be added
		 *           more attribute(s) and value(s) depending on the result.
		 * @param   _pByRange will have True or False value depending on whether range is desired.
		 */

	public invoiceRequestHandler()
	{
		//set amount format
		amtF.setMaximumFractionDigits(3);
		amtF.setGroupingUsed(false);
	}

	public void generateHotBillByAccounts(Document inDoc, Document outDoc, boolean _pByRange) {

			/* Signature:-->

				public Vector createHotBill(String[] _pAccounts, String _pStartDate, String _pEndDate);
				public Vector createHotBill(String _pStartAccount, String _pEndAccounts,String _pStartDate, String _pEndDate);

			*/

				Vector _resultVector;
				int _totalHotBills ;

				Element inRoot  = inDoc.getDocumentElement();
				Element outRoot = outDoc.getDocumentElement();

				if (_pByRange == false) {
						NodeList _nList = ((Node)inRoot).getChildNodes();
						int _noOfActs =  _nList.getLength();

						String[] _accIds = new String[_noOfActs];

						for(int i=0; i < _noOfActs; i++)
							_accIds[i] = (String) ((Element)(_nList.item(i))).getAttribute("ID");

						invoiceBean _oInvoiceBean = new invoiceBean();
						_resultVector = _oInvoiceBean.createHotBill(_accIds);
				 }
				 else
					{
						String _startAccId = inRoot.getAttribute("StartAccountId");
						String _endAccId = inRoot.getAttribute("EndAccountId");
						invoiceBean _oInvoiceBean = new invoiceBean();
						_resultVector = _oInvoiceBean.createHotBill(_startAccId,_endAccId);
					}

	        	_totalHotBills = _resultVector.size();

				for(int i = 0; i < _totalHotBills; i++)
					{	String[] actAttr = (String[])(_resultVector.elementAt(i));
						Element te = outDoc.createElement("HotBillByAccounts");
						outRoot.appendChild(te);
						//te.setAttribute("Id", actAttr[0]);
						te.setAttribute("Id", actAttr[7]);
						te.setAttribute("AccountNo", " ");
						te.setAttribute("Name", " ");
						te.setAttribute("AccountId", actAttr[2]);
						te.setAttribute("TimeStamp", actAttr[11]);
						//te.setAttribute("Amount", ""+Math.abs(Double.valueOf(actAttr[4]).doubleValue()));
						String amt = amtF.format(Double.valueOf(actAttr[4]).doubleValue());
						te.setAttribute("Amount", amt);
						te.setAttribute("IspvnId", actAttr[3]);
						te.setAttribute("TransType", actAttr[1]);
						te.setAttribute("Note", actAttr[5]);
						te.setAttribute("Status", actAttr[6]);
						te.setAttribute("ReferenceId", actAttr[7]);
						te.setAttribute("Operator", actAttr[8]);
						te.setAttribute("Description", actAttr[9]);
						te.setAttribute("CcType", actAttr[10]);
						te.setAttribute("CheckNo", actAttr[12]);
						te.setAttribute("StartBillPeriod", actAttr[13]);
						te.setAttribute("EndBillPeriod", actAttr[14]);

					// _sqlFetch = _sqlFetch.append("SELECT id, trans_type, account_id, ispvn_id, amount, note, status, reference_id,");
					// _sqlFetch = _sqlFetch.append("operator, description, cc_type, to_char(TIMESTAMP,'dd-MON-yyyy') timestamp,  check_no, ");
					// _sqlFetch = _sqlFetch.append(" start_billperiod,end_billperiod ");
					// _sqlFetch = _sqlFetch.append("  FROM Transaction  WHERE ");

					}
			}

	/**
	 * This document calls the Invoice Bean's reInvoiceByAccounts() to do re-invoicing.
	 * @return   void. The outDoc  xml document will have result after calling the
	 *           underlying Invoice Bean method
	 * @param    inDoc an xml document which describes Accounts for doing re-invoice.
	 * @param    outDoc an empty xml document that will be passed-in and will be added
	 *           more attribute(s) and value(s) depending on the result.
	 * @param   _pByRange will have True or False value depending on whether range is desired.
	 */


	public void reInvoiceByAccounts(Document inDoc, Document outDoc, boolean _pByRange) {

		/* Signature

		    Vector reInvoiceByIsp(String _pIspvn, String _pStartDate, String _pEndDate);
			Vector reInvoiceByAccounts(String[] _pAccounts, String _pStartDate, String _pEndDate);
			Vector reInvoiceByAccounts(String _pStartAccount, String _pEndAccounts,String _pStartDate, String _pEndDate);
		*/


					Vector _resultVector;
					int _totalReInvoices ;

					Element inRoot  = inDoc.getDocumentElement();
					Element outRoot = outDoc.getDocumentElement();

					String _startDate = inRoot.getAttribute("StartDate");
					String _endDate = inRoot.getAttribute("EndDate");

					if (_pByRange == false) {
							NodeList _nList = ((Node)inRoot).getChildNodes();
							int _noOfActs =  _nList.getLength();

							String[] _accIds = new String[_noOfActs];

							for(int i=0; i < _noOfActs; i++)
								_accIds[i] = (String) ((Element)(_nList.item(i))).getAttribute("ID");

							invoiceBean _oInvoiceBean = new invoiceBean();
							_resultVector = _oInvoiceBean.reInvoiceByAccounts(_accIds, _startDate, _endDate);
					 }
					 else
						{
							String _startAccId = inRoot.getAttribute("StartAccountId");
							String _endAccId = inRoot.getAttribute("EndAccountId");
							invoiceBean _oInvoiceBean = new invoiceBean();
							_resultVector = _oInvoiceBean.reInvoiceByAccounts(_startAccId,_endAccId,_startDate, _endDate);
						}

					_totalReInvoices = _resultVector.size();

					for(int i = 0; i < _totalReInvoices; i++)
						{	String[] actAttr = (String[])(_resultVector.elementAt(i));
							Element te = outDoc.createElement("ReInvoiceByAccounts");
							outRoot.appendChild(te);
							//te.setAttribute("Id", actAttr[0]);
							te.setAttribute("Id", actAttr[7]);
							te.setAttribute("AccountNo", " ");
							te.setAttribute("Name", " ");
							te.setAttribute("AccountId", actAttr[2]);
							te.setAttribute("TimeStamp", actAttr[11]);
							//te.setAttribute("Amount", ""+Math.abs(Double.valueOf(actAttr[4]).doubleValue()));
							String amt = amtF.format(Double.valueOf(actAttr[4]).doubleValue());
							te.setAttribute("Amount", amt);
							te.setAttribute("IspvnId", actAttr[3]);
							te.setAttribute("TransType", actAttr[1]);
							te.setAttribute("Note", actAttr[5]);
							te.setAttribute("Status", actAttr[6]);
							te.setAttribute("ReferenceId", actAttr[7]);
							te.setAttribute("Operator", actAttr[8]);
							te.setAttribute("Description", actAttr[9]);
							te.setAttribute("CcType", actAttr[10]);
							te.setAttribute("CheckNo", actAttr[12]);
							te.setAttribute("TransTerms", actAttr[13]);
							te.setAttribute("CcNum", actAttr[14]);
							te.setAttribute("CcExpMonth", actAttr[15]);
							te.setAttribute("CcExpYear", actAttr[16]);
							te.setAttribute("CheckDate", actAttr[17]);
							te.setAttribute("CheckBank", actAttr[18]);
						}

	}


	/**
	 * This document calls the Invoice Bean's getInvoicesByIsp() to do invoicing.
	 * @return   The parameters are call by reference. The outDoc is the xml document
	 *           which will be returned as a result of calling underlying Bean
	 * @param    inDoc an xml document which describes necessary attriubutes for creating invoicing.
	 * @param    outDoc an empty xml document that will be passed-in and will be added
	 *           more attribute(s) and value(s) depending on the result.
	 * @param   _pByRange will have True or False value depending on whether range is desired.

	 */

	public void getInvoicesByIsp(Document inDoc, Document outDoc) {

			/* Signature:-->
					Vector getInvoicesByIsp(String _pIspvn, String _pStartDate,
					                        String _pEndDate);
			*/

			Vector _resultVector;
			int _totalInvoices ;

			Element inRoot  = inDoc.getDocumentElement();
			Element outRoot = outDoc.getDocumentElement();

			String _ispvnId = inRoot.getAttribute("IspvnId");
			String _startDate = inRoot.getAttribute("StartDate");
			String _endDate = inRoot.getAttribute("EndDate");
			String _exportReq =  inRoot.getAttribute("ExportRequest");
			String _exportFile =  inRoot.getAttribute("ExportFile");
			String _headerString;

			invoiceBean _oInvoiceBean = new invoiceBean();
			_resultVector = _oInvoiceBean.getInvoicesByIsp(_ispvnId,_startDate,_endDate);


			_totalInvoices = _resultVector.size();

			if (_exportReq != null) {
				_headerString = "Id,TransType,AccountId,IspvnId,Amount,Note,Status," +
				       "ReferenceId,Operator,Description,CcType,TimeStamp,CheckNo";
				try {
                       createCSVFile(_resultVector, _exportFile, _headerString); // call the function to create a csv file
				    }
				 catch(Exception ex) {
				   		System.out.println("An exception while creating the file");
				   	}
  			}
			else
			{
				for(int i = 0; i < _totalInvoices; i++)
					{	String[] invoiceAttr = (String[])(_resultVector.elementAt(i));
						Element te = outDoc.createElement("InvoiceByIsp");
						outRoot.appendChild(te);
						//te.setAttribute("Id", invoiceAttr[0]);
						te.setAttribute("Id", invoiceAttr[7]);
						te.setAttribute("TransType", invoiceAttr[1]);
						te.setAttribute("AccountId", invoiceAttr[2]);
						te.setAttribute("IspvnId", invoiceAttr[3]);
						te.setAttribute("Amount", invoiceAttr[4]);
						te.setAttribute("Note", invoiceAttr[5]);
						te.setAttribute("Status", invoiceAttr[6]);
						te.setAttribute("ReferenceId", invoiceAttr[7]);
						te.setAttribute("Operator", invoiceAttr[8]);
						te.setAttribute("Description", invoiceAttr[9]);
						te.setAttribute("CcType", invoiceAttr[10]);
						te.setAttribute("TimeStamp", invoiceAttr[11]);
						te.setAttribute("CheckNo", invoiceAttr[12]);
					}
			}
		}

		// This function helps to create  comma seperated csv file
        // which will be called whenever there is a need for exporting

        private void createCSVFile(Vector _resultVector, String _outPutFile, String _headerString) throws IOException {

			int _totalInvoices;
			int _totalAttributes;
			StringBuffer _outputString = new StringBuffer("");

			System.out.println("Trying to create file" + _outPutFile);
			File _oFileName = new File(_outPutFile);
			try { System.out.println("Delete Result: "+_oFileName.delete()); } catch(Exception e) { System.out.println("Error Deleting: "+e.toString()); }
			RandomAccessFile  _rFile = new RandomAccessFile(_oFileName,"rw");

			_totalInvoices = _resultVector.size();
			_rFile.writeBytes(_headerString);

			for(int i = 0; i < _totalInvoices; i++) {
				String[] invoiceAttr = (String[])(_resultVector.elementAt(i));
				_totalAttributes = invoiceAttr.length - 2;
				  for(int j = 0; j < _totalAttributes; j++) {
					_outputString = _outputString.append(invoiceAttr[j]);
					_outputString = _outputString.append(",");
				}
					_outputString = _outputString.append(invoiceAttr[_totalAttributes]+"\n");
					_rFile.writeBytes((String) (_outputString.toString()));
					_outputString.setLength(0);
			}
			_rFile.close();
		}

	/**
			 * This document calls the Invoice bean's getInvoicesByAccounts() to get Invoice details.
			 * @return   void. The outDoc  xml document will have result after calling the
			 *           underlying Invoice Bean method
			 * @param    inDoc an xml document which describes necessary attriubutes for calling the Bean.
			 * @param    outDoc an empty xml document that will be passed-in and will be added
			 *           more attribute(s) and value(s) depending on the result.
			 * @param   _pByRange will have True or False value depending on whether range is desired.
	 */

		public void getInvoicesByAccounts(Document inDoc, Document outDoc, boolean _pByRange) {

			/* Signature:-->
					Vector getInvoicesByAccounts(String[] _pAccounts, String _pStartDate,
														  String _pEndDate);
					Vector getInvoicesByAccounts(String _pStartAccount, String _pEndAccount,
														  String _pStartDate, String _pEndDate);
			*/

			Vector _resultVector;
			int _totalInvoices ;

			Element inRoot  = inDoc.getDocumentElement();
			Element outRoot = outDoc.getDocumentElement();

			String _startDate = inRoot.getAttribute("StartDate");
			String _endDate = inRoot.getAttribute("EndDate");
			String _exportReq =  inRoot.getAttribute("ExportRequest");
			String _exportFile =  inRoot.getAttribute("ExportFile");
			_exportFile = "/Jws/public_html/portal/Invoices.zip";

			String _headerString;


			if (_pByRange == false) {
					NodeList _nList = ((Node)inRoot).getChildNodes();
					int _noOfActs =  _nList.getLength();

					String[] _accIds = new String[_noOfActs];

					for(int i=0; i < _noOfActs; i++)
						_accIds[i] = (String) ((Element)(_nList.item(i))).getAttribute("ID");

					invoiceBean _oInvoiceBean = new invoiceBean();
					_resultVector = _oInvoiceBean.getInvoicesByAccounts(_accIds, _startDate, _endDate);
			 }
			 else
				{
					String _startAccId = inRoot.getAttribute("StartAccountId");
					String _endAccId = inRoot.getAttribute("EndAccountId");
					invoiceBean _oInvoiceBean = new invoiceBean();
					_resultVector = _oInvoiceBean.getInvoicesByAccounts(_startAccId,_endAccId,_startDate, _endDate);
				}

			_totalInvoices = _resultVector.size();

			//if (_exportReq != null) {
				_headerString = "Id,TransType,AccountId,IspvnId,Amount,Note,Status," +
				               " ReferenceId,Operator,Description,CcType,TimeStamp,CheckNo\n";
				try {
			           createCSVFile(_resultVector, _exportFile, _headerString); // call the function to create a csv file
					}
				catch(Exception ex) {
					   System.out.println("An exception while creating the file");
					}
			  	//}
				//else
				//  {
					for(int i = 0; i < _totalInvoices; i++)
					{	String[] actAttr = (String[])(_resultVector.elementAt(i));
						Element te = outDoc.createElement("InvoiceByAccounts");
						outRoot.appendChild(te);
						//te.setAttribute("Id", actAttr[0]);
						te.setAttribute("Id", actAttr[7]);
						te.setAttribute("AccountNo", " ");
						te.setAttribute("Name", " ");
						te.setAttribute("AccountId", actAttr[2]);
						te.setAttribute("TimeStamp", actAttr[11]);
						//te.setAttribute("Amount", ""+Math.abs(Double.valueOf(actAttr[4]).doubleValue()));
						String amt = amtF.format(Double.valueOf(actAttr[4]).doubleValue());
						te.setAttribute("Amount", amt);
						te.setAttribute("IspvnId", actAttr[3]);
						te.setAttribute("Note", actAttr[5]);
						te.setAttribute("Status", actAttr[6]);
						te.setAttribute("ReferenceId", actAttr[7]);
						te.setAttribute("Operator", actAttr[8]);
						te.setAttribute("Description", actAttr[9]);
						te.setAttribute("CcType", actAttr[10]);
						te.setAttribute("TransType", actAttr[1]);
						te.setAttribute("CheckNo", actAttr[12]);
				}
 		}

	/**
			 * This document calls the Invoice bean's getInvoiceDetail() to get Invoice details.
			 * @return   void. The outDoc  xml document will have result after calling the
			 *           underlying Invoice Bean method
			 * @param    inDoc an xml document which describes necessary attriubutes for calling the Bean.
			 * @param    outDoc an empty xml document that will be passed-in and will be added
			 *           more attribute(s) and value(s) depending on the result.
	 */

			// This returns the details of the given Invoice number from Orders (Order_Line/Order_Item_Details
			// table..) which we need to figure out.
			public void getInvoiceDetail(Document inDoc, Document outDoc) {

				/* public Vector getInvoiceDetail(String _pInvoiceID); */

					Vector _resultVector;
					int _totalRows ;

					Element inRoot  = inDoc.getDocumentElement();
					Element outRoot = outDoc.getDocumentElement();


					String _invoiceId = inRoot.getAttribute("InvoiceId");
					String _exportReq =  inRoot.getAttribute("ExportRequest");
					String _exportFile =  inRoot.getAttribute("ExportFile");

					String _headerString;

					NodeList _nList = ((Node)inRoot).getChildNodes();
					int _noOrders =  _nList.getLength();
					StringBuffer _allInvoiceIds = new StringBuffer("");

					String[] _orderIds = new String[_noOrders];

					if (_noOrders == 1) {
						_allInvoiceIds = _allInvoiceIds.append((String) ((Element)(_nList.item(0))).getAttribute("ID"));
					}
					else
					  {
						for(int i=0; i < _noOrders; i++)
							_orderIds[i] = (String) ((Element)(_nList.item(i))).getAttribute("ID");

						for(int i=0; i < _orderIds.length-2; i++) {
							_allInvoiceIds = _allInvoiceIds.append(_orderIds[i]);
							_allInvoiceIds = _allInvoiceIds.append(",");
							}
						_allInvoiceIds = _allInvoiceIds.append(_orderIds.length-1);
				      }

					invoiceBean _oInvoiceBean = new invoiceBean();
					_resultVector = _oInvoiceBean.getInvoiceDetail(_allInvoiceIds.toString());

					_totalRows = _resultVector.size();

					if (_exportReq != null) {
						_headerString = "Description,OrderId,ItemId,Minutes,Amount,ItemGroup ";
						try {
					         createCSVFile(_resultVector, _exportFile, _headerString); // call the function to create a csv file
							 }
						catch(Exception ex) {
						System.out.println("An exception while creating the file");
						}
					  }
					else  {
						for(int i = 0; i < _totalRows; i++)
						{	String[] invoiceAttr = (String[])(_resultVector.elementAt(i));
							Element te = outDoc.createElement("InvoiceDetail");
							outRoot.appendChild(te);
							te.setAttribute("Description", invoiceAttr[0]);
							te.setAttribute("OrderId", invoiceAttr[1]);
							te.setAttribute("ItemId", invoiceAttr[2]);
							te.setAttribute("Minutes", invoiceAttr[3]);
							te.setAttribute("Amount", invoiceAttr[4]);
							te.setAttribute("ItemGroup", invoiceAttr[5]);
						}
					}
				}
		}