import java.awt.*;
import java.net.URL;
import java.util.Calendar;
import java.text.SimpleDateFormat;

/**
This class works as a glue to bind all the disparate classes to work
together and share resources. This class defines all the global variables
and constants, and provides methods to reference the shared Objects in the
Billing Portal Application.
*/
public class AppConfig
{
	//Color Definitions
	static final Color Color_Browser = Color.gray;
	static final Color Color_AppBar  = new Color(224, 198, 0);
	static final Color Color_Client  = Color.lightGray;
	static final Color Color_ModuleTitleBar = new Color(168, 168, 168);

	//Predefined fonts
	static final Font Font_11 = new Font("SansSerif", Font.PLAIN, 11);
	static final Font Font_12 = new Font("SansSerif", Font.PLAIN, 12);
	static final Font Font_13 = new Font("SansSerif", Font.PLAIN, 13);
	static final Font Font_14 = new Font("SansSerif", Font.PLAIN, 14);
	static final Font Font_B11 = new Font("SansSerif", Font.BOLD, 11);
	static final Font Font_B12 = new Font("SansSerif", Font.BOLD, 12);
	static final Font Font_B13 = new Font("SansSerif", Font.BOLD, 13);
	static final Font Font_B14 = new Font("SansSerif", Font.BOLD, 14);
	static final Font Font_ModuleTitle = new Font("SansSerif", Font.BOLD, 16);

	//Cursors
	static final Cursor Cursor_HAND    = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
	static final Cursor Cursor_DEFAULT = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

	//Layout Definitions
	static final int TotalWidth   = 1280;
	static final int TotalHeight  = 780;
	static final int ClientWidth  = 1130;
	static final int ClientIndent = 150;
	static final int AppBarWidth  = 140;
	static final int SepBarWidth  = 10;
	static final int ModuleTop    = 60;
	static final int DividerTop   = 70;
	static final int CustPanelTop    = 0;
	static final int CustPanelIndent = 150;
	static final int CustPanelHeight = 53;

	static String portalURL=null;	//Reference to The Billing POrtal URL
	static String operatorID="0";	//Reference to OperatorID

	static BillingApplet billingApplet=null;	//Reference to the Main Applet
	static AppBar billingAppBar=null;			//Reference to AppBar
	static AppModule billingModule=null;		//Reference to active Module
	static CustomerPanel customerPanel=null;	//Reference to the shared Customer Panel

	static String[] ispvnNameList = new String[0];	//Reference to Ispvn Names array
	static String[] ispvnIDList = new String[0];	//Reference to Ispvb IDs array
	static int curIspvnIndex = 0;					//Reference to current ISP

	//Reference to the shared HttpCommunicator Object
	static HttpCommunicator httpCommunicator = new HttpCommunicator();

	//Reference to the shared XMLParser
	static XMLParser xmlParser = new XMLParser();

	//Cached XML Documents
	static Document[] DocumentCache = new Document[16];
	static long[] DocumentFetchTime = new long[16];
	static final long   CacheTimeout = 300000;

	//Cached Documents ID
	static final int DOC_ACCOUNT_ID    = 0;
	static final int DOC_ACCOUNT_FNAME = 1;
	static final int DOC_ACCOUNT_LNAME = 2;

	//Variables to enable Java-to-Javascript Messaging
	static final int SHOW_HTML   = 101;
	static final int CLOSE_HTML  = 102;
	static final int EXPORT_DATA = 103;
	static int htmlMessage=SHOW_HTML;
	static final String htmlHeader = "<html><head><title>Billing Portal Html Viewer</title></head><body bgcolor=white>";
	static String htmlData = "";

	//Claendar and dateFormat Object to manipulates dates
	static Calendar dCalendar = Calendar.getInstance();
	static SimpleDateFormat dFormat = new SimpleDateFormat ("MM/dd/yyyy");

	/**
	This method reinitializes all static varibles to cut memory effect of
	browser's JavaVM which doesn't release the objects and caches them
	*/
	public static void reset()
	{
		billingApplet=null;
		billingAppBar=null;
		billingModule=null;

		portalURL=null;
		ispvnNameList = new String[0];
		ispvnIDList   = new String[0];
		curIspvnIndex = 0;

		DocumentCache = new Document[16];
		DocumentFetchTime = new long[16];
	}

	/** Sets a reference to the main Billing Applet
	@param bApp Reference to the Billing Applet
	*/
	public static void setBillingApplet(BillingApplet bApp) { billingApplet = bApp; }
	/** Gets a reference to the main Billing Applet
	@return reference to the main Billing Applet
	*/
	public static BillingApplet getBillingApplet() { return billingApplet; }

	/** Sets a reference to the AppBar Object
	@param tbar Reference to AppBar Object
	*/
	public static void setAppBar(AppBar tbar) { billingAppBar = tbar; }
	/** Gets a reference to the AppBar Object
	@return reference to the AppBar Object
	*/
	public static AppBar getAppBar() { return billingAppBar; }

	/** Sets the currently active Module
	@param tMod Refernece to the curently Active Module
	*/
	public static void setActiveModule(AppModule tMod)	{ billingModule = tMod; }
	/** Gets reference to the currently active Module
	@return reference to the currently active Module
	*/
	public static AppModule getActiveModule() { return billingModule; }

	/** Sets a reference to the shared Customer Panel
	@param tp Refernece to the CustomerPanel Object
	*/
	public static void setCustomerPanel(CustomerPanel tp) { customerPanel = tp; }
	/** Gets a reference to the shared CustomerPanel
	@return reference to the shared CustomerPanel
	*/
	public static CustomerPanel getCustomerPanel() { return customerPanel; }

	/** Gets a refernece to the Shared HttpCommunicator Object
	@return refernece to the Shared HttpCommunicator Object
	*/
	public static HttpCommunicator getCommunicator() { return httpCommunicator; }

	/** Gets a refernece to the Shared XMLParser
	@return refernece to the Shared XMLParser
	*/
	public static XMLParser getParser() { return xmlParser; }

	/**
	Sets the Html data which will be used by Javascript to dynamically generate
	the Html content
	@param the Html data which is interpreted accouding to the Html Message
	*/
	public static void setHtmlData(String hData) { htmlData = hData; }
	/**
	This method is used by Javascript code to get the dynamic html and display
	it in a new Browser Window.
	@return an Html document to be dislpyed in a new Window
	*/
	public static String getHtmlData() { return htmlHeader+htmlData+"</body></html>"; }
	/**
	This is the method which works as a bridge for Java to Javascript messaging.<p>
	If the hMsg parameter equals SHOW_HTML, a document AppMessage.html is loaded in
	a new window. That document uses Javascript to call getHtmlData() method and
	and displays the Html Data dynamically into the browser window.<p>
	If the hMsg parameter equals EXPORT_DATA, then a new Document is opened in a
	new window and the type of doument is one of the export types e.g. CSV or ZIP.
	@param hMsg an intger specifying the message code. Can be either SHOW_HTML or EXPORT_DATA
	*/
	public static void sendHtmlMessage(int hMsg)
	{
		htmlMessage = hMsg;
		try {
		if(htmlMessage == SHOW_HTML)
			billingApplet.appContext.showDocument(new URL(billingApplet.getDocumentBase(),"AppMessage.html"), "tmpf0101xxyy0");
		else if(htmlMessage == EXPORT_DATA)
			billingApplet.appContext.showDocument(new URL(billingApplet.getDocumentBase(),htmlData), "tmpf0101xxyy0");
		} catch (Exception ex) { System.out.println("URL Error: "+ex.toString()); }
	}
	/**
	Gets the message code as a string
	@return the string representation of the Html Mssage Code
	*/
	public static String getHtmlMessage()
	{
		if(htmlMessage == SHOW_HTML) return "SHOW_HTML";
		else if(htmlMessage == CLOSE_HTML) return "CLOSE_HTML";
		return "NONE";
	}

	//Date Utils
	/**
	Returns the system date in mm/dd/yyyy format. This method is used in conjunction
	with adjustDate method. After getting the date, adjustDate can modify parts of it.
	If the reset param is false, the previous instance of date with all the adjustments
	is returned. It reset param is true, a new instance of system date is returned and
	all the adjustments and calculations to it are lost.
	@param reset a boolean to indicate whether to get a fresh instance of date or use the earlier one
	@return the date in mm/dd/yyyy format
	*/
	public static String getDate(boolean reset)
	{
		if(reset) dCalendar = Calendar.getInstance();
		return dFormat.format(dCalendar.getTime());
	}

	/**
	This method adds a value to a specific part of date to create new dates.
	@param reset a boolean to indicate whether to get a fresh instance of date or to use the previous one
	@param dpart a constan as defined in Calendar class to indicate which part of date is to be modified
	@param amt amount by which the date part is to be modified. e.g amt of 7 days will increment the date by one week
	*/
	public static void adjustDate(boolean reset, int dpart, int amt)
	{
		if(reset) dCalendar = Calendar.getInstance();
		dCalendar.add(dpart, amt);
	}
}
