/**
This interface is implemented by all Billing Modules
and serves as a messaging gateway between a module and
the rest of the application
*/
public interface AppModule
{
	//Module IDs
	public static final short INVOICES=0;
	public static final short PAYMENTS=1;
	public static final short ADJUSTMENTS=2;
	public static final short REPORTS=3;
	public static final short TOOLS=4;

	/**
	Gets an array of Module Names
	@return the module names in string array
	*/
	public static final String[] moduleName = {  "Invoices", "Payments", "Adjustments",
																"Reports", "Tools" };
	/**
	Gets the unique ID asociated with a module.
	@return a hort value representing the ID of this module
	*/
	public short getModuleID();

	/**
	Gets the Container Object (a Panel) associated with the module to add GUI objects
	@return a Panel which acts as the Module container
	*/
	public java.awt.Container getModuleContainer();

	/**
	Activates or deactivates a module. Called by BillingApplet. To switch to a different
	module, the BillingApplet calls this method with state=false to deactive the current
	module and calls the new module with state=true to make that module the currently
	active module
	@param a boolean to indicate whether to activate or deactivate a module
	*/
	public void setActive(boolean state);

	/**
	Gives the module an opportunity to reset itself. Called by Billing Applet
	*/
	public void reset();
}
