import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

/**
BillingApplet is the main GUI Applet form the Billing Portal Application.
The Applet space is divided into two parts: the left side contains an AppBar
which hosts the Main Menu and Ispvn List Box to make selections from. The
right side is used for loading the desired module. Every time a user makes
a selection from the main menu, the specific module (called AppModule) Panel
is added into the Applet and if there was another module selected previously,
its Panel is removed. The Applet loads the modules dynamically, that is, only
those modules which are selected by user are downloaded. The BillingApplet is
also the main communication gateway for all Java-to-Javascript communications.
*/
public class BillingApplet extends java.applet.Applet
{
	java.applet.AppletContext appContext;

	private static AppBar pAppBar;
	private AppModule invoicesModule;
	private AppModule paymentsModule;
	private AppModule adjustmentsModule;
	private AppModule reportsModule;
	private AppModule toolsModule;

	public void init()
	{
		//reset all global objects
		AppConfig.reset();

		this.setLayout(null);
		this.setSize(AppConfig.TotalWidth, AppConfig.TotalHeight);
		//this.setBackground(Color.lightGray);
		this.setBackground(Color.gray);

		//get Applet Context
		appContext = this.getAppletContext();

		//set reference to this applet
		AppConfig.setBillingApplet(this);

		//Initialize Messaging for this Applet. Must do this before adding
		//any component to Applet.
		XToolkit.initMessaging(this);
		//XToolkit.showWaitMsg("Loading. Please wait..........");

		//Add AppBar to hold Main App Menu and Misc stuff
		pAppBar = new AppBar();
		this.add(pAppBar);
		AppConfig.setAppBar(pAppBar);	//set it for global referencing

		//Add Separator between Modules and AppBar
		Canvas tc = new Canvas();
		tc.setBounds(AppConfig.AppBarWidth, 0, AppConfig.SepBarWidth, AppConfig.TotalHeight);
		tc.setBackground(AppConfig.Color_Browser);
		this.add(tc);

		//set customer panel
		CustomerPanel cPanel = new CustomerPanel();
		this.add(cPanel);
		AppConfig.setCustomerPanel(cPanel);
	}

	/**
	This method is called by AppBar when user makes a slection for a particular module.
	@param mName a string specifying the module name as defined in AppModule class
	*/
	public void setModule(String mName)
	{
		//if portal URL not specified, return
		if(AppConfig.portalURL == null) return;
		//set the customer panel size to it minimum size
		AppConfig.getCustomerPanel().setBounds(AppConfig.ClientIndent, 0, AppConfig.ClientWidth, AppConfig.CustPanelHeight);
		if(mName.equals(AppModule.moduleName[AppModule.INVOICES])) loadInvoicesModule();
		else if(mName.equals(AppModule.moduleName[AppModule.PAYMENTS])) loadPaymentsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.ADJUSTMENTS])) loadAdjustmentsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.REPORTS])) loadReportsModule();
		else if(mName.equals(AppModule.moduleName[AppModule.TOOLS])) loadToolsModule();
		else return;
	}

	/**
	This method is called by AppBar when user select a particular Ispvn.
	@param ispIndex a index into the array of currently accessible Ispvns
	*/
	public void setIspvn(int ispIndex)
	{	//set the current ispIndex to the supplied value
		AppConfig.curIspvnIndex = ispIndex;
		//reset the Customer Panel to flush out all the customer data for previous Ispvn
		AppConfig.getCustomerPanel().reset();
		AppModule activeModule = AppConfig.getActiveModule();
		//Reset the Actie Module because the change in Ispvn makes all the prev data inconsistent
		if(activeModule != null) activeModule.reset();
	}

	private void loadInvoicesModule()
	{
		//Show wait message box
		//XToolkit.showWaitMsg("Loading. Please wait..........");
//System.out.println("Going to sleep......");
//		try { Thread.sleep(5000); } catch(Exception e) { }
//System.out.println("Unhiding it......");
		//XToolkit.hideWaitMsg();

		//Get reference to currently active module
		AppModule cApp = AppConfig.getActiveModule();
		if(invoicesModule == null)
		{	//Dynamically load the module as per need basis
			invoicesModule = ModuleLoader.getInvoicesModule();
			this.add(invoicesModule.getModuleContainer());
		}
		else if(cApp == invoicesModule) return;		//User just clicked the same module
		if(cApp != null) cApp.setActive(false);		//Deactive the current module
		invoicesModule.setActive(true);				//Set the selected module as the active one
	}

	private void loadPaymentsModule()
	{
		//see the documentaion of loadInvoicesModule() method for all the logic---
		AppModule cApp = AppConfig.getActiveModule();
		if(paymentsModule == null)
		{
			paymentsModule = ModuleLoader.getPaymentsModule();
			this.add(paymentsModule.getModuleContainer());
		}
		else if(cApp == paymentsModule) return;
		if(cApp != null) cApp.setActive(false);
		paymentsModule.setActive(true);
	}

	private void loadAdjustmentsModule()
	{
		//see the documentaion of loadInvoicesModule() method for all the logic---
		AppModule cApp = AppConfig.getActiveModule();
		if(adjustmentsModule == null)
		{
			adjustmentsModule = ModuleLoader.getAdjustmentsModule();
			this.add(adjustmentsModule.getModuleContainer());
		}
		else if(cApp == adjustmentsModule) return;
		if(cApp != null) cApp.setActive(false);
		adjustmentsModule.setActive(true);
	}

	private void loadReportsModule()
	{
		//see the documentaion of loadInvoicesModule() method for all the logic---
		AppModule cApp = AppConfig.getActiveModule();
		if(reportsModule == null)
		{
			reportsModule = ModuleLoader.getReportsModule();
			this.add(reportsModule.getModuleContainer());
		}
		else if(cApp == reportsModule) return;
		if(cApp != null) cApp.setActive(false);
		reportsModule.setActive(true);
	}

	private void loadToolsModule()
	{
		//see the documentaion of loadInvoicesModule() method for all the logic---
		AppModule cApp = AppConfig.getActiveModule();
		if(toolsModule == null)
		{
			toolsModule = ModuleLoader.getToolsModule();
			this.add(toolsModule.getModuleContainer());
		}
		else if(cApp == toolsModule) return;
		if(cApp != null) cApp.setActive(false);
		toolsModule.setActive(true);
	}

	//Communication with JavaScript
	/**
	Called by Javascript to set the Portal URL
	@param u Portal URL in absolute form
	*/
	public void setPortalURL(String u) { AppConfig.portalURL = u; AppConfig.getCommunicator().setURL(u); }

	/**
	This method is called by Javascript to add Ispvn Names accessible to the current user.
	Javascript calls this method multiple times, because the Javascript to Java Integration
	supported by most browsers doesn't allow passing parameters other than primtive Java
	types, that is, int, String etc. So Javascript can't pass arrays and must call this
	method muliple times.
	@param iN the Ispvn Name to be added in the Ispvn Name List
	*/
	public void addIspvnName(String iN)
	{
		//Rebuild the Ispvn Names array
		int tLen = AppConfig.ispvnNameList.length;
		String[] tnList = new String[tLen+1];
		if(tLen > 0) System.arraycopy(AppConfig.ispvnNameList, 0, tnList, 0, tLen);
		tnList[tLen] = iN;
		AppConfig.ispvnNameList = tnList;
	}
	/**
	This method is called by Javascript to add Ispvn IDs accessible to the current user.
	Javascript calls this method multiple times, because the Javascript to Java Integration
	supported by most browsers doesn't allow passing parameters other than primtive Java
	types, that is, int, String etc. So Javascript can't pass arrays and must call this
	method muliple times.
	@param iD the Ispvn ID to be added in the Ispvn ID List
	*/
	public void addIspvnID(String iD)
	{
		//Rebuild the Ispvn IDs array
		int tLen = AppConfig.ispvnIDList.length;
		String[] tiList = new String[tLen+1];
		if(tLen > 0) System.arraycopy(AppConfig.ispvnIDList, 0, tiList, 0, tLen);
		tiList[tLen] = iD;
		AppConfig.ispvnIDList  = tiList;
	}

	/**
	Yet another workaround to enable Javascript to finally add the Ispvn List
	into the AppBar. Because Javascript can't use arrays as parameters when it calls
	Java methods, it first builds the Ispvn Names and IDs arrays using the addIspvnName()
	and addIspvnID() methods, and when its done building, calls this method to indicate
	the end of list and to load the List into the AppBar
	*/
	public void refreshIspvnData() { pAppBar.refreshISPList(); }

	/**
	Called by Javascript to set the Operator ID which is required in the Transaction Logs.
	@param opId the Operator ID as a string
	*/
	public void setOperatorID(String opId) { AppConfig.operatorID = opId; }

	/**
	To be called by Javascript to get the BillingApplet Message as a String
	@return the String representing the BillingApplet Message
	*/
	public String getAppMessage() { return AppConfig.getHtmlMessage(); }

	/**
	To be called by Javascript to get Html document as a String to create a document dynamically.
	@ return the Html document to be displayed in a new browswer window
	*/
	public String getHtmlString() { return AppConfig.getHtmlData(); }


	//This is just a Test Method - is not used by application
	public void setCommApplet(Object co)
	{
		System.out.println("set Applet Called...");
		//java.applet.Applet ca = (java.applet.Applet)co;
		System.out.println(co);
		java.util.Enumeration eApp = appContext.getApplets();
		System.out.println(eApp);
		while(eApp.hasMoreElements())
		{
			Object ta = eApp.nextElement();
			System.out.println(ta);
		}
		//if(ca == null) System.out.println("Null Applet"); else System.out.println("Not Null !");//else ca.TriggerComm();
	}

	//This is just a Test Method - is not used by application
	public String testJS()	//should be removed later...
	{
		String msg = "Portal URL: "+AppConfig.portalURL;
		msg = msg.concat("\nIspvnList Length: "+AppConfig.ispvnNameList.length);
		return msg;
	}
}
