import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Vector;

/**
This class implements all the Font-End functionality of Invoices Module
*/
public class InvoicesApp extends Panel implements AppModule, ActionListener, ItemListener, HttpListener
{
	private Color appBtnColor  = new Color(228, 228, 228);	//color of invoices menu buttons
	private Color actvTableColor = new Color(228, 228, 228);//color of grid

	Panel  appPanel = new Panel();	//All the forms are drawn here
	XLayout xLayout = new XLayout(appPanel);	//XLayout handles all the forms layout
	//Results Table
	XGrid actvTable;

	private Document actvDoc=null;	//reference to the response XMl document
	private Element invRow;	//the currently selected row from the invoices table

	//Invoices form Components
	Label appTitleLb, appText1Lb, appText2Lb;
	Label cCodeLb, cCodeValLb, cNameLb, cNameValLb, cTypeLb, cTypeValLb, cBalLb, cBalValLb;
	Label periodLb, sDateLb, eDateLb, pDateLb, pAmtLb, pTermsLb;
	Label cardTypeLb, cardNoLb, cardMonthLb, cardYearLb;
	Label checkNoLb, checkDateLb, checkBankLb, remarkLb;
	TextField sDateTx, eDateTx, pDateTx, pAmtTx, cardNoTx, cardMonthTx, cardYearTx;
	TextField checkNoTx, checkDateTx, checkBankTx, remarkTx;
	Choice periodList, pTermsList, cardTypeList;
	Checkbox accMultiple = new Checkbox("Multiple Accounts");
	Button appBtn;

	//Form Components Array
	Component[] appComponent;

	//Current Module Otpion
	String cAPPOption = "";

	//Define constants for form layout
	static final int INVBTN    = 99;

	static final int itemStartIndex = 100;	//marks the begining of form Items
	static final int TITLE     = 100;
	static final int TEXT1     = 101;
	static final int TEXT2     = 102;
	static final int ACTVTABLE = 103;
	static final int ACCMULTI  = 104;
	//static final int CCODEVAL  = 105;
	//static final int CNAME     = 106;
	//static final int CNAMEVAL  = 107;
	//static final int CTYPE     = 108;
	//static final int CTYPEVAL  = 109;
	//static final int CBAL      = 110;
	//static final int CBALVAL   = 111;

	static final int labeledItemStartIndex = 112;	//marks the begining of labeled Items
	static final int PERIOD    = 112;
	static final int SDATE     = 114;
	static final int EDATE     = 116;
	static final int PDATE     = 118;
	static final int AMOUNT    = 120;
	static final int PTERMS    = 122;
	static final int CARDTYPE  = 124;
	static final int CARDNO    = 126;
	static final int CARDMONTH = 128;
	static final int CARDYEAR  = 130;
	static final int CHECKNO   = 132;
	static final int CHECKDATE = 134;
	static final int CHECKBANK = 136;
	static final int REMARK    = 138;

	//Imvoices Application Options
	private static final String[] appOptions = { "List Invoices", "Hot Bill", "Re-Invoice"};
	//Buttons for Invoices Forms
	private static final String[] appButtons = { "Get Invoices", "Generate Hot Bill",
																"Re Invoice", "OK", "Cancel", "Export",
																"Print", "E-mail" };
	//Title Labels for various forms
	private static final String[] appText    = { "Customer Invoices", "Customer Hot Bills",
																"Customer Re-Invoices" };
	//This defines forms as per XLayout specifications
	private static final int[][][] appForms =
	{
		{ // 0 - List Invoices
			{ TITLE, 0, 180 },
			{ XLayout.VGAP, 20 },
			{ XLayout.INDENT, 70 },
			{ XLayout.ROWDEF, 24, 0, 0, 30, 4 },
			{ XLayout.COLDEF, 240, 215 },
			{ XLayout.LDEF, 110, 95 },
			{ PERIOD, ACCMULTI },
			{ SDATE, EDATE },
			{ XLayout.ROWDEF, 24, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 110, 120 },
			{ XLayout.SKIPCOL },
			{ INVBTN, 0 }

		},
		{ // 1 - List Invoices Results
			{ TEXT1, 0, 250 },
			{ XLayout.VGAP, 20 },
			{ XLayout.ROWDEF, 181, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 560 },
			{ ACTVTABLE },
			{ XLayout.ROWDEF, 15 },
			{ XLayout.SKIPROW },
			{ XLayout.ROWDEF, 24, 0, 0, 4, 2 },
			{ XLayout.COLDEF, 120, 120, 120 },
			{ INVBTN, 5 },
			{ INVBTN, 6 }
		},
		{ // 2 - Hot Bill
			{ TITLE, 1, 180 },
			{ XLayout.VGAP, 20 },
			{ XLayout.INDENT, 70 },
			{ XLayout.ROWDEF, 24, 0, 0, 30, 4 },
			{ XLayout.COLDEF, 240, 215 },
			{ XLayout.LDEF, 110, 95 },
			{ PERIOD, ACCMULTI },
			{ SDATE, EDATE },
			{ XLayout.ROWDEF, 24, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 110, 120 },
			{ XLayout.SKIPCOL },
			{ INVBTN, 1 }

		},
		{ // 3 - Hot Bill Results
			{ TEXT1, 0, 250 },
			{ XLayout.VGAP, 20 },
			{ XLayout.ROWDEF, 180, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 560 },
			{ ACTVTABLE },
			{ XLayout.ROWDEF, 15 },
			{ XLayout.SKIPROW },
			{ XLayout.ROWDEF, 24, 0, 0, 4, 2 },
			{ XLayout.COLDEF, 120, 120, 120 },
			{ INVBTN, 5 },
			{ INVBTN, 6 }
		},
		{ // 4 - Re-Invoices
			{ TITLE, 2, 180 },
			{ XLayout.VGAP, 20 },
			{ XLayout.INDENT, 70 },
			{ XLayout.ROWDEF, 24, 0, 0, 30, 4 },
			{ XLayout.COLDEF, 240, 215 },
			{ XLayout.LDEF, 110, 95 },
			{ PERIOD, ACCMULTI },
			{ SDATE, EDATE },
			{ XLayout.ROWDEF, 24, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 110, 120 },
			{ XLayout.SKIPCOL },
			{ INVBTN, 2 }

		},
		{ // 5 - Re-Invoices Results
			{ TEXT1, 0, 250 },
			{ XLayout.VGAP, 20 },
			{ XLayout.ROWDEF, 180, 0, 0, 0, 0 },
			{ XLayout.COLDEF, 560 },
			{ ACTVTABLE },
			{ XLayout.ROWDEF, 15 },
			{ XLayout.SKIPROW },
			{ XLayout.ROWDEF, 24, 0, 0, 4, 2 },
			{ XLayout.COLDEF, 120, 120, 120 },
			{ INVBTN, 5 },
			{ INVBTN, 6 }
		}
	};


	/**
	The default constructor creates Menu Buttons, Froms Panel and all the widgets
	for form layout.
	*/
	public InvoicesApp()
	{
		//No thanks to Java Layouts. We are better off working with our own layout
		this.setLayout(null);
		this.setBounds(AppConfig.ClientIndent, AppConfig.ModuleTop, AppConfig.ClientWidth, AppConfig.TotalHeight);
		this.setBackground(AppConfig.Color_Client);

		int btnTop = 32;	//Top y coordinate for Invoices Menu Buttons
		int btnWidth = 150, btnHeight = 24;
		for(int i=0; i < appOptions.length; i++)
		{
		XButton	btn1 = new XButton(appOptions[i]);	//Create diamond shaped buttons
		btn1.setBounds(20+(btnWidth+5)*i, btnTop, btnWidth, btnHeight);
		btn1.setColor(appBtnColor);
		btn1.addActionListener(this);
		btn1.setActionCommand(appOptions[i]);
		this.add(btn1);
		}

		//add appPanel
		appPanel.setBounds(20, AppConfig.DividerTop+15, AppConfig.ClientWidth, 400);
		appPanel.setBackground(AppConfig.Color_Client);
		this.add(appPanel);

		/*actvTable = new XTable();
		actvTable.setBackground(AppConfig.Color_ModuleTitleBar);
		actvTable.setForeground(actvTableColor);
		actvTable.setBounds(20, 40, 500, 200);
		//actvTable.setHeaderWidth(new int[] { 50, 100, 150, 150 });
		//actvTable.setHeaderText(new String[] { "Type", "Trn ID", "Trn Date", "Amount" });
		actvTable.setVisible(false);
		appPanel.add(actvTable);*/

		actvTable = new XGrid();
		actvTable.setBounds(20, AppConfig.DividerTop+40, 620, 201);
		actvTable.setContentBackground(AppConfig.Color_ModuleTitleBar);
		actvTable.setContentForeground(actvTableColor);
		//actvTable.setDataFont(AppConfig.Font_B12);
		actvTable.setPageSize(7);
		actvTable.displayPageTabs(true);
		actvTable.setColumns(new int[]{50, 100, 150, 150});
		actvTable.setHeader(new String[]{ "Type", "Trn ID", "Trn Date", "Amount" });
		actvTable.setColProperty(0, XGrid.PROP_TEXTCOLOR, Color.red);
		actvTable.setColProperty(0, XGrid.PROP_TEXTFONT, AppConfig.Font_B12);
		actvTable.setColProperty(0, XGrid.PROP_DATALINK, "Y");
		actvTable.setLinkListener(this, "LINK_SELECTED");
		actvTable.setVisible(false);
		appPanel.add(actvTable);

		//define gui elements for various payment options
		defineFormElements();
	}

	/**
	This method instantiates all form widgets for the applications and initializes
	them. No size or location is specified at this time because XLayout will
	set them at the time of displaying a form. The controls defined here can be
	shared among many forms.
	*/
	void defineFormElements()
	{
		appTitleLb = new Label();
		appTitleLb.setFont(AppConfig.Font_ModuleTitle);
		appTitleLb.setBackground(AppConfig.Color_ModuleTitleBar);

		appText1Lb = new Label();
		appText1Lb.setFont(AppConfig.Font_ModuleTitle);
		appText1Lb.setBackground(AppConfig.Color_ModuleTitleBar);
		appText2Lb = new Label();
		appText2Lb.setFont(AppConfig.Font_B12);
		appText2Lb.setForeground(Color.gray);
		//appText2Lb.setBackground(AppConfig.Color_ModuleTitleBar);

		periodLb = new Label("Period : ");
		periodLb.setFont(AppConfig.Font_12);

		sDateLb = new Label("From (mm/dd/yyyy): ");
		sDateLb.setFont(AppConfig.Font_12);
		eDateLb = new Label("To (mm/dd/yyyy): ");
		eDateLb.setFont(AppConfig.Font_12);

		cCodeLb = new Label("Customer Code:");
		cCodeLb.setFont(AppConfig.Font_B12);
		cCodeLb.setBackground(actvTableColor);
		cCodeValLb = new Label("1121");
		cCodeValLb.setFont(AppConfig.Font_B12);
		cCodeValLb.setBackground(actvTableColor);
		cCodeValLb.setForeground(Color.red);
		cNameLb = new Label(" Name:");
		cNameLb.setFont(AppConfig.Font_B12);
		cNameLb.setBackground(actvTableColor);
		cNameValLb = new Label("Joo Lee");
		cNameValLb.setFont(AppConfig.Font_B12);
		cNameValLb.setBackground(actvTableColor);
		cNameValLb.setForeground(Color.red);
		cTypeLb = new Label(" Type:");
		cTypeLb.setFont(AppConfig.Font_B12);
		cTypeLb.setBackground(actvTableColor);
		cTypeValLb = new Label("Prepaid");
		cTypeValLb.setFont(AppConfig.Font_B12);
		cTypeValLb.setBackground(actvTableColor);
		cTypeValLb.setForeground(Color.red);
		cBalLb = new Label(" Balance:");
		cBalLb.setFont(AppConfig.Font_B12);
		cBalLb.setBackground(actvTableColor);
		cBalValLb = new Label("200.00");
		cBalValLb.setFont(AppConfig.Font_B12);
		cBalValLb.setBackground(actvTableColor);
		cBalValLb.setForeground(Color.red);

		pDateLb = new Label("Date(mm/dd/yyyy): ");
		pAmtLb  = new Label("Amount: ");
		pTermsLb= new Label("Payment Terms:");
		cardTypeLb = new Label("Credit Card: ");
		cardNoLb = new Label("Card No:");
		cardMonthLb = new Label("Expiry Month:");
		cardYearLb = new Label("Expiry Year:");
		checkNoLb = new Label("Check No: ");
		checkDateLb = new Label("Check Date:");
		checkBankLb = new Label("Check Bank:");
		remarkLb = new Label("Remarks:");

		sDateTx = new TextField();
		eDateTx = new TextField();
		pDateTx = new TextField();
		pAmtTx  = new TextField();
		cardNoTx = new TextField();
		cardMonthTx = new TextField();
		cardYearTx = new TextField();
		checkNoTx = new TextField();
		checkDateTx = new TextField();
		checkBankTx = new TextField();
		remarkTx = new TextField();

		periodList = new Choice();
		periodList.addItemListener(this);
		periodList.add("Specify"); periodList.add("30 Days"); periodList.add("60 Days");
		periodList.add("90 Days"); periodList.add("180 Days");
		pTermsList = new Choice();
		pTermsList.add("CASH"); pTermsList.add("CREDIT");
		pTermsList.add("CHECK"); pTermsList.add("OTHER");
		cardTypeList = new Choice();
		cardTypeList.add("VISA"); cardTypeList.add("MASTERCARD");
		cardTypeList.add("AMEX"); cardTypeList.add("DISCOVER");
		cardTypeList.add("DINNERS"); cardTypeList.add("OTHER");

		appBtn = new Button();
		appBtn.setBackground(AppConfig.Color_ModuleTitleBar);
		appBtn.addActionListener(this);

		//fill the form components array
		appComponent = new Component[]
		{	appTitleLb, appText1Lb, appText2Lb, actvTable, accMultiple, cCodeValLb,
			cNameLb, cNameValLb, cTypeLb, cTypeValLb, cBalLb, cBalValLb,
			periodLb, periodList, sDateLb, sDateTx, eDateLb, eDateTx,
			pDateLb, pDateTx, pAmtLb, pAmtTx, pTermsLb, pTermsList,
			cardTypeLb, cardTypeList, cardNoLb, cardNoTx,
			cardMonthLb, cardMonthTx, cardYearLb, cardYearTx,
			checkNoLb, checkNoTx, checkDateLb, checkDateTx,
			checkBankLb, checkBankTx, remarkLb, remarkTx
		};
	}

	public void paint(Graphics g)
	{
		g.setColor(AppConfig.Color_ModuleTitleBar);
		g.fillRect(0, 0, AppConfig.ClientWidth, 20);
		XToolkit.setColorScheme(XToolkit.COLORSCHEME_SOFT3D);
		XToolkit.draw3DBox(g, 0, 0, AppConfig.ClientWidth+1, 21, true);
		g.setColor(Color.white);
		g.setFont(AppConfig.Font_ModuleTitle);
		g.drawString(moduleName[INVOICES], 5, 16);
		XToolkit.setColorScheme(XToolkit.COLORSCHEME_SHARP3D);
		XToolkit.draw3DLineH(g, 0, AppConfig.DividerTop, AppConfig.ClientWidth+1, 1, false);
	}


	//AppModule interface methods
	public short getModuleID() { return AppModule.INVOICES; }

	public Container getModuleContainer() { return this; }

	public void setActive(boolean state)
	{
		if(state)
		{	//CustomerPanel tp = AppConfig.getCustomerPanel();
			//tp.setSelectionMode(false);
			//this.add(tp, 0);
			AppConfig.setActiveModule(this);
			AppConfig.getCustomerPanel().setVisible(true);
			AppConfig.getCustomerPanel().setBounds(AppConfig.ClientIndent, 0, AppConfig.ClientWidth, AppConfig.CustPanelHeight);
		}
		this.setVisible(state);
	}

	//this resets the module and clears the module area
	public void reset()
	{
		xLayout.reset();
	}

	//itemlistener interface
	public void itemStateChanged(ItemEvent e)
	{
		//int newIndex = ISPList.getSelectedIndex();
		String pText = periodList.getSelectedItem();
		if(pText.equals("Specify")) return;
		eDateTx.setText(AppConfig.getDate(true));
		if(pText.equals("30 Days")) AppConfig.adjustDate(false, java.util.Calendar.DATE, -30);
		else if(pText.equals("60 Days")) AppConfig.adjustDate(false, java.util.Calendar.DATE, -60);
		else if(pText.equals("90 Days")) AppConfig.adjustDate(false, java.util.Calendar.DATE, -90);
		else if(pText.equals("180 Days")) AppConfig.adjustDate(false, java.util.Calendar.DATE, -180);
		sDateTx.setText(AppConfig.getDate(false));
	}

	//ActionListener interface
	public void actionPerformed(ActionEvent e)
	{
		String cOption = e.getActionCommand();
		//System.out.println("Button: "+cOption+"  cPMTOption: "+cPMTOption);

		if(cOption.equals("LINK_SELECTED") || cOption.equals("Print"))
		{
			int row, col;
			if(cOption.equals("Print")) row = -1;
			else { row = e.getID()/100;  col = e.getID() % 100; }
			getInvoiceDetails(row);
			//AppConfig.setHtmlData(getInvoiceDetails(row));
			//AppConfig.sendHtmlMessage(AppConfig.SHOW_HTML);
			return;
		}


		if(cOption.equals("List Invoices"))
		{	if(AppConfig.getCustomerPanel().hasCustomers())
			{	cAPPOption = cOption;
				layoutForm(0);
			}
		}
		else if(cOption.equals("Hot Bill"))
		{	if(AppConfig.getCustomerPanel().hasCustomers())
			{	cAPPOption = cOption;
				layoutForm(2);
			}
		}
		else if(cOption.equals("Re-Invoice"))
		{	if(AppConfig.getCustomerPanel().hasCustomers())
			{	cAPPOption = cOption;
				layoutForm(4);
			}
		}
		else if(cOption.equals("Get Invoices")) getInvoices();
		else if(cOption.equals("Generate Hot Bill")) generateHotBill(3);
		else if(cOption.equals("Re Invoice")) reInvoice();
		else if(cOption.equals("Cancel")) xLayout.reset();
		else if(cOption.equals("Export"))
		{	AppConfig.setHtmlData("Invoices.zip");
			AppConfig.sendHtmlMessage(AppConfig.EXPORT_DATA);
		}
	}

	//handle data from HttpCommunicator
	public void receiveHttpData(Object data, int dataID)
	{
		if(dataID == 101)	//Get Invoices
		{
			XToolkit.hideProgressMessage();
			Document doc = (Document)data;
			actvDoc = doc;
			if(doc == null) return;
			displayInvoices(doc);
		}
		else if(dataID == 102)	//Hot Bill
		{
			XToolkit.hideProgressMessage();
			Document doc = (Document)data;
			actvDoc = doc;
			if(doc == null) return;
			displayInvoices(doc);
		}
		else if(dataID == 103)	//Re-Invoice
		{
			XToolkit.hideProgressMessage();
			Document doc = (Document)data;
			actvDoc = doc;
			if(doc == null) return;
			displayInvoices(doc);
		}
		else if(dataID == 104)	//Invoice Details
		{
			XToolkit.hideProgressMessage();
			Document doc = (Document)data;
			if(doc == null) return;
			displayInvoiceDetails(doc);
		}
	}

	void getInvoices()
	{
		layoutForm(1);	//receipts results
		XToolkit.setProgressListener(AppConfig.getCommunicator());
		XToolkit.showProgressMessage("Getting Customer Invoices. Please wait......");
		XMLParser parser = AppConfig.getParser();
		actvTable.setVisible(false);
		String sDate = sDateTx.getText();
		String eDate = eDateTx.getText();
		if(sDate.length() < 1) sDate = "*";
		if(eDate.length() < 1) eDate = "*";
		Document reqDoc = parser.createDocument("Request");
		Element root = reqDoc.getDocumentElement();
		root.setAttribute("ID", ""+ReqDef.INVOICESBYACCOUNTS);
		root.setAttribute("StartDate", sDate);
		root.setAttribute("EndDate", eDate);
		if(accMultiple.getState())	//if multiple accounts selected
		{	String[][] cList = AppConfig.getCustomerPanel().getSelectedCustomers();
			int cItems = cList.length;
			for(int i=0; i < cItems; i++)
			{	Element te = reqDoc.createElement("AccountId");
				root.appendChild(te);
				te.setAttribute("ID", cList[i][1]);
			}
		}
		else
		{	Element te = reqDoc.createElement("AccountId");
			root.appendChild(te);
			te.setAttribute("ID", (AppConfig.getCustomerPanel().getCustomerData())[1]);
		}
		AppConfig.getCommunicator().processXMLRequest(reqDoc, this, 101);
	}

	public void getInvoiceDetails(int rowId)
	{
		StringBuffer htmlBuf = new StringBuffer();
		Element root = actvDoc.getDocumentElement(), tRow;
		NodeList tList = root.getChildNodes();
		int tLen = tList.getLength();

		if(rowId > -1)
		{	tRow  = (Element)(root.getChildNodes().item(rowId));
			invRow = tRow;
			//htmlBuf.append("<H1>Invoice No: "+tRow.getAttribute("Id")+"<br><H3>Account No: "+tRow.getAttribute("AccountId")+"</H3><H3>Amount: "+tRow.getAttribute("Amount")+"</H3><br><br>");
			//return htmlBuf.toString();
			XToolkit.setProgressListener(AppConfig.getCommunicator());
			XToolkit.showProgressMessage("Getting Customer Invoices. Please wait......");
			XMLParser parser = AppConfig.getParser();
			Document reqDoc = parser.createDocument("Request");
			Element root2 = reqDoc.getDocumentElement();
			root2.setAttribute("ID", ""+ReqDef.INVOICEDETAILS);
			root2.setAttribute("InvoiceId", tRow.getAttribute("Id"));
			Element te = reqDoc.createElement("InvoiceId");
			root2.appendChild(te);
			te.setAttribute("ID", tRow.getAttribute("Id"));
			AppConfig.getCommunicator().processXMLRequest(reqDoc, this, 104);
			return;
		}

		for(int i=0; i < tLen; i++)
		{	tRow  = (Element)(root.getChildNodes().item(i));
			htmlBuf.append("<H1>Invoice No: "+tRow.getAttribute("Id")+"<br><H3>Account No: "+tRow.getAttribute("AccountId")+"</H3><H3>Amount: "+tRow.getAttribute("Amount")+"</H3><br><br>");
			AppConfig.setHtmlData(htmlBuf.toString());
			AppConfig.sendHtmlMessage(AppConfig.SHOW_HTML);
		}

		//return htmlBuf.toString();
	}

	public void displayInvoiceDetails(Document doc)
	{
		StringBuffer htmlBuf = new StringBuffer();
		Element root = doc.getDocumentElement(), tRow;
		NodeList tList = root.getChildNodes();
		int tLen = tList.getLength();
		String[] cdata = AppConfig.getCustomerPanel().getCustomerData();

		htmlBuf.append("<H1>Invoice No: "+invRow.getAttribute("Id")+"<br><H4>Account No: "+cdata[0]+"<br><H4>Name: "+cdata[2]+" "+cdata[3]+"<br><H4>Amount: "+invRow.getAttribute("Amount")+"<br><br>");
		htmlBuf.append("<UL><H3>Calls Summary:</H3>");
		htmlBuf.append("<Table bgcolor=#E4E4E4 HSPACE=0 VSPACE=0 BORDER=1 CELLPADDING=2 CELLSPACING=0>");
		htmlBuf.append("<tr><th width=80>Item ID</th><th width=260>Description</th><th width=80>Quantity</th><th width=80>Amount</th></tr>");

		for(int i=0; i < tLen; i++)
		{	tRow  = (Element)(tList.item(i));

			htmlBuf.append("<tr><td>"+tRow.getAttribute("ItemId")+"</td><td>"+tRow.getAttribute("Description")+"</td><td>"+tRow.getAttribute("Minutes")+"</td><td>"+tRow.getAttribute("Amount")+"</td><tr>");
		}
		htmlBuf.append("</table><p>All call duration in seconds</UL>");

		AppConfig.setHtmlData(htmlBuf.toString());
		AppConfig.sendHtmlMessage(AppConfig.SHOW_HTML);
	}


	void generateHotBill(int formNo)
	{
		layoutForm(formNo);	//activities results
		XToolkit.setProgressListener(AppConfig.getCommunicator());
		XToolkit.showProgressMessage("Generating Hot Bills. Please wait......");
		XMLParser parser = AppConfig.getParser();
		actvTable.setVisible(false);
		String sDate = sDateTx.getText();
		String eDate = eDateTx.getText();
		if(sDate.length() < 1) sDate = "*";
		if(eDate.length() < 1) eDate = "*";
		Document reqDoc = parser.createDocument("Request");
		Element root = reqDoc.getDocumentElement();
		root.setAttribute("ID", ""+ReqDef.HOTBILLBYACCOUNTS);
		root.setAttribute("StartDate", sDate);
		root.setAttribute("EndDate", eDate);
		if(accMultiple.getState())	//if multiple accounts selected
		{	String[][] cList = AppConfig.getCustomerPanel().getSelectedCustomers();
			int cItems = cList.length;
			for(int i=0; i < cItems; i++)
			{	Element te = reqDoc.createElement("AccountId");
				root.appendChild(te);
				te.setAttribute("ID", cList[i][1]);
			}
		}
		else
		{	Element te = reqDoc.createElement("AccountId");
			root.appendChild(te);
			te.setAttribute("ID", (AppConfig.getCustomerPanel().getCustomerData())[1]);
		}
		AppConfig.getCommunicator().processXMLRequest(reqDoc, this, 102);
	}


	void reInvoice()
	{
		layoutForm(5);	//Reinvoice results
		XToolkit.setProgressListener(AppConfig.getCommunicator());
		XToolkit.showProgressMessage("Re-Generating Invoices. Please wait......");
		XMLParser parser = AppConfig.getParser();
		actvTable.setVisible(false);
		String sDate = sDateTx.getText();
		String eDate = eDateTx.getText();
		if(sDate.length() < 1) sDate = "*";
		if(eDate.length() < 1) eDate = "*";
		Document reqDoc = parser.createDocument("Request");
		Element root = reqDoc.getDocumentElement();
		root.setAttribute("ID", ""+ReqDef.REINVOICESBYACCOUNTS);
		root.setAttribute("StartDate", sDate);
		root.setAttribute("EndDate", eDate);
		if(accMultiple.getState())	//if multiple accounts selected
		{	String[][] cList = AppConfig.getCustomerPanel().getSelectedCustomers();
			int cItems = cList.length;
			for(int i=0; i < cItems; i++)
			{	Element te = reqDoc.createElement("AccountId");
				root.appendChild(te);
				te.setAttribute("ID", cList[i][1]);
			}
		}
		else
		{	Element te = reqDoc.createElement("AccountId");
			root.appendChild(te);
			te.setAttribute("ID", (AppConfig.getCustomerPanel().getCustomerData())[1]);
		}
		AppConfig.getCommunicator().processXMLRequest(reqDoc, this, 103);
	}

	void displayInvoices(Document actvDoc)
	{
		actvTable.setColumns(new int[] { 80, 80, 170, 80, 100, 100 });
		actvTable.setHeader(new String[] { "Invoice No", "Account No", "Name", "ID", "Date", "Amount" });
		actvTable.setColProperty(0, XGrid.PROP_TEXTCOLOR, Color.red);
		actvTable.setColProperty(0, XGrid.PROP_TEXTFONT, AppConfig.Font_B12);
		actvTable.setColProperty(0, XGrid.PROP_DATALINK, "Y");
		actvTable.setLinkListener(this, "LINK_SELECTED");
		actvTable.loadDocument(actvDoc);
		//reset the data with account_no and name
		String[][] gdata = actvTable.getText();
		CustomerPanel cPanel = AppConfig.getCustomerPanel();
		String[][] cdata = cPanel.getSelectedCustomers();
		String pCustID = "-1.01";
		int pCI = -1;
		for(int i=0; i < gdata[0].length; i++)
		{	if(!gdata[3][i].equals(pCustID))
			{ pCustID = gdata[3][i];	pCI = cPanel.getCustomerIndex(pCustID); }
			gdata[1][i] = cdata[pCI][0];
			String Name = "";
			if(!cdata[pCI][2].equals("null")) Name = Name+cdata[pCI][2];
			if(!cdata[pCI][3].equals("null")) Name = Name+" "+cdata[pCI][3];
			gdata[2][i] = Name;
		}
		//actvTable.setText(gdata);
		actvTable.setVisible(true);
	}


	void layoutForm(int fID)
	{
		xLayout.reset();

		//layout each item
		int fItems = appForms[fID].length;
		for(int i=0; i < fItems; i++) layoutXItem(appForms[fID][i]);
	}

	void layoutXItem(int[] itemData)
	{
		if(xLayout.handleXItem(itemData)) return;	//if xLayout item

		int itemLen = itemData.length;
		for(int i=0; i < itemLen; i++)
		{
			int itemID  = itemData[i];
			//handle TITLE and TEXT items here only
			if(itemID == TITLE || itemID == TEXT1)
			{	xLayout.setRowProperties(1, new int[] {20});
				xLayout.setColProperties(1, new int[] { itemData[i+2] });
				appTitleLb.setBounds(xLayout.getXBounds());
				if(itemID == TITLE)
				{	appTitleLb.setBackground(AppConfig.Color_ModuleTitleBar);
					appTitleLb.setText(appOptions[itemData[i+1]]);
				}
				else
				{	appTitleLb.setBackground(AppConfig.Color_Client);
					appTitleLb.setText(appText[itemData[i+1]]);
				}
				appPanel.add(appTitleLb);
				return;
			}
			//handle form buttons
			if(itemID == INVBTN)
			{	appBtn = new Button();
				appBtn.setBackground(AppConfig.Color_ModuleTitleBar);
				appBtn.addActionListener(this);
				appBtn.setLabel(appButtons[itemData[i+1]]);
				appBtn.setActionCommand(appButtons[itemData[i+1]]);
				appBtn.setBounds(xLayout.getXBounds());
				//appBtn.setBounds(200, 200, 120, 22);
				appPanel.add(appBtn);
				return;
			}

			//Let XLayout handle rest of the items
			int itemIndex = itemID - itemStartIndex;
			Component txComp = appComponent[itemIndex];
			if(itemID < labeledItemStartIndex) //handle non-labeled component
				xLayout.addComponent(appComponent[itemIndex]);
			else
			{	xLayout.addLabeledComponent(appComponent[itemIndex], appComponent[itemIndex+1]);
				txComp = appComponent[itemIndex+1];
			}
			if(txComp instanceof TextField) ((TextField)txComp).setCaretPosition(0);
		}
	}

}
