import java.awt.event.*;

/**
A simple implementation of a non-blocking timer in Java. Java doesn't
have an in-built timer class and the only to elapse time is to call
Thread.sleep() But Thread.sleep() blocks the application for a specified
duration of time. This class implements a simple non-blocking timer. Any
class using this timer should implement ActionListener interface and call
addActionListener method of timer. When timer has elapsed specified amount
time, it will send an action event to the calling object and actionCommand
will be: TIMER
*/
public class Timer extends Thread
{
	int intervalMillis=100;
	ActionListener timerListener=null;

	/**
	Creates a timer which sleeps for specified amount of milliseconds
	@param millis the sleep time in milliseconds
	*/
	public Timer(int millis)
	{	super();
		intervalMillis = millis;
	}

	/**
	Users of this class must call this method or else the timer will have
	no Object to notify to at the end of the sleep interval.
	@param a an ActionListener to whom the time will send an action command
	*/
	public void addActionListener(ActionListener a)
	{
		timerListener = a;
	}

	/**
	Sets the sleep intervel for timer
	*/
	public void setInterval(int millis)
	{
		intervalMillis = millis;
	}

	/**
	This starts the timer thread. The user of this class must call the start()
	method on timer objects so that the Tread will start running.
	*/
	public void run()
	{
		try {
			while(true)
			{
			this.sleep(intervalMillis);
			timerListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "TIMER"));
			}
		} catch (InterruptedException e) { }
	}
}
