import java.awt.*;
import java.awt.event.*;

public class XButton extends Canvas implements MouseListener
{
	static final int TOTAL_POINTS=8;
	Polygon XPolygon;
	private String LabelText="";
	int[] pX = new int[TOTAL_POINTS];
	int[] pY = new int[TOTAL_POINTS];
	int[] bX = new int[4];
	int[] bY = new int[4];
	int[] dX = new int[4];
	int[] dY = new int[4];
	int gradient=0;
	int cw, ch;
	int textX, textY;

	static final Font btnFont = new Font("SansSerif", Font.BOLD, 11);
	static final FontMetrics btnFM = Toolkit.getDefaultToolkit().getFontMetrics(btnFont);

	static Color brightEdgeColor = Color.white;
	static Color darkEdgeColor = Color.gray;
	static Color shadowColor   = Color.black;
	Color topEdgeColor  = brightEdgeColor;
	Color botUEdgeColor = darkEdgeColor;
	Color botLEdgeColor = shadowColor;
	Color btnNormalColor = Color.yellow;
	Color btnFocusColor  = Color.lightGray;
	Color curbtnColor = btnNormalColor;
	boolean sizeDefined   = false;
	boolean buttonPressed = false;

	ActionListener btnActionListener=null;
	String actionCmd = "OK";

	public XButton()
	{
		super();
		//setBackground(Color.red);
		//setVisible(true);
		this.addMouseListener(this);
	}

	public XButton(String Ltx)
	{
		super();
		LabelText = Ltx;
		//setBackground(Color.red);
		//setVisible(true);
		this.addMouseListener(this);
	}

	public void setColor(Color c)
	{
		curbtnColor = btnNormalColor = c;
		this.repaint();
	}

	public void setFocusColor(Color f)
	{
		btnFocusColor = f;
	}

	public void setLabel(String sL)
	{
		LabelText = sL;
	}

	public void addActionListener(ActionListener aL)
	{
		btnActionListener = aL;
	}

	public void setActionCommand(String aC)
	{
		actionCmd = aC;
	}

	public void setSize(int width, int height)
	{
		super.setSize(width, height);
		gradient = (height/2)-1;
		cw = width;  ch = height;
		calculatePoints();
	}

	public void setSize(Dimension d)
	{
		this.setSize(d.width, d.height);
	}

	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x, y, width, height);
		cw = width;  ch = height;
		gradient = (height/2)-1;
		calculatePoints();
		//for(int i = 0; i < TOTAL_POINTS; i++)
		//System.out.println(i+": X: "+pX[i]+" Y: "+pY[i]);
	}

	public void setBounds(Rectangle r)
	{
		this.setBounds(r.x, r.y, r.width, r.height);
	}

	public void calculatePoints()
	{
		//set fill polygon points
		pX[1] = gradient;  pX[2] = cw - gradient - 1;
		pX[3] = cw - 1;  pX[4] = pX[3];
		pX[5] = pX[2];   pX[6] = pX[1];
		pY[0] = gradient + 1;
		pY[1] = (2*gradient) + 1;  pY[2] = pY[1];
		pY[3] = pY[0];  pY[4] = gradient;
		pY[7] = pY[4];

		//set bright edge points
		bX[0] = pX[4];  bY[0] = pY[4];
		bX[1] = pX[5];  bY[1] = pY[5];
		bX[2] = pX[6];  bY[2] = pY[6];
		bX[3] = pX[7];  bY[3] = pY[7];

		//set dark edge points
		dX[0] = pX[0]+1;  dY[0] = pY[0];
		dX[1] = pX[1];  dY[1] = pY[1]-1;
		dX[2] = pX[2];  dY[2] = pY[2]-1;
		dX[3] = pX[3];  dY[3] = pY[3]-1;

		//create a polygon object for mouseClick accuracy
		XPolygon = new Polygon(pX, pY, TOTAL_POINTS);

		//Define the Location of Label
		textX = (cw-btnFM.stringWidth(LabelText))/2;
		textY = ch - (ch-5)/2;
	}

	public void paint(Graphics g)
	{
		g.setColor(curbtnColor);
		g.fillPolygon(pX, pY, TOTAL_POINTS);
		g.setColor(botLEdgeColor);
		g.drawPolyline(pX, pY, 4);
		g.setColor(botUEdgeColor);
		g.drawPolyline(dX, dY, 4);
		g.setColor(topEdgeColor);
		g.drawPolyline(bX, bY, 4);
		g.setColor(shadowColor);
		g.setFont(btnFont);
		g.drawString(LabelText, textX, textY);
	}


	//------ Implementation of MouseListener Interface------
	//------------------------------------------------------
	public void mouseClicked(MouseEvent e)
	{
		//System.out.println("MouseClicked");
	}

	public void mouseEntered(MouseEvent e)
	{
		//System.out.println("mouseEntered");
		curbtnColor = btnFocusColor;
		this.repaint();
	}

	public void mouseExited(MouseEvent e)
	{
		//System.out.println("mouseExited");
		curbtnColor = btnNormalColor;
		this.repaint();
	}

	public void mousePressed(MouseEvent e)
	{
		if(!XPolygon.contains(e.getPoint())) return;
		topEdgeColor  = shadowColor;
		botUEdgeColor = brightEdgeColor;
		botLEdgeColor = this.getBackground();
		textY++;
		buttonPressed = true;
		this.repaint();
	}

	public void mouseReleased(MouseEvent e)
	{
		if(!buttonPressed) return;
		topEdgeColor  = brightEdgeColor;
		botUEdgeColor = darkEdgeColor;
		botLEdgeColor = shadowColor;
		textY--;
		buttonPressed = false;
		this.repaint();
		if(XPolygon.contains(e.getPoint()) && btnActionListener != null)
		btnActionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCmd));
	}

}
