import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class XTable extends Panel implements ActionListener, MouseListener
{
	private Font  headerFont   = new Font("SansSerif", Font.PLAIN, 13);
	private Font  dataFont     = new Font("SansSerif", Font.PLAIN, 12);
	private Font  hrefFont     = new Font("SansSerif", Font.PLAIN, 12);
	private FontMetrics dataFM = Toolkit.getDefaultToolkit().getFontMetrics(dataFont);
	private FontMetrics hrefFM = Toolkit.getDefaultToolkit().getFontMetrics(dataFont);
	private int fontHeight = dataFM.getHeight();
	private int hrefFontHeight = hrefFM.getHeight();

	private Color headerColor = new Color(168, 168, 168);
	private Color tableColorB = new Color(224, 198, 0);
	private Color tableColorF = Color.lightGray;
	private Color dataColor   = Color.black;
	private Color hrefColor   = Color.red;
	private Color hrefColorH  = Color.blue;

	private int   headerHeight  = 21;
	private int   headerCols    = 0;
	private int[] headerColW;
	private Button[] headerBtn;

	private int tableWidth=0, tableHeight=0;
	private int contentWidth=0, contentHeight=0;
	private int borderT = 1;

	//Layout specific
	private int cRowHeight = 20;
	private int cRowMLeft  = 0;
	private int cRowMTop   = 0;
	private int cRowMRight = 0;
	private int cRowMBot   = 0;
	private int cColCount  = 0;
	private int[] cColWidth;
	private int[] cColMLeft;
	private int[] cColMTop;
	private int[] cColMRight;
	private int[] cColMBot;
	private int[] cColWidthM;
	private int   hrefCol = -1;

	//Cursor specific
	private int cursorPos = 0;
	private int cursorX   = 0;
	private int cursorY   = 0;

	ScrollPane scrollP;
	Panel contentP;


	public XTable()
	{
		super();
		scrollP  = new ScrollPane();
		contentP = new Panel();
		initXTable();
	}

	public XTable(int sb)
	{
		super();
		scrollP  = new ScrollPane(sb);
		contentP = new Panel();
		initXTable();
	}

	private void initXTable()
	{
		this.setLayout(null);
		contentP.setBackground(tableColorB);
		contentP.setLayout(null);
		scrollP.add(contentP);
		scrollP.getHAdjustable().setUnitIncrement(10);
		scrollP.getVAdjustable().setUnitIncrement(10);
		super.add(scrollP);
	}

	public void setBackground(Color c)
	{
		tableColorB = c;
		contentP.setBackground(c);
	}

	public void setForeground(Color c) { tableColorF = c; }

	public void setColor(Color c) { dataColor = c; }

	public void setFont(Font f)
	{
		dataFont = f;
		dataFM = Toolkit.getDefaultToolkit().getFontMetrics(f);
		fontHeight = dataFM.getHeight();
	}

	public void setBorderThickness(int t) { borderT = t; }

	public void setSize(int width, int height)
	{
		super.setSize(width, height);
		tableWidth = width;  tableHeight = height;
		contentWidth  = width-4;
		contentHeight = (headerCols > 0) ? height-headerHeight-4 : height-4;
		scrollP.setSize(width, contentHeight+4);
		contentP.setSize(contentWidth, contentHeight);
	}

	public void setSize(Dimension d)
	{
		this.setSize(d.width, d.height);
	}

	public void setBounds(int x, int y, int width, int height)
	{
		super.setBounds(x, y, width, height);
		tableWidth = width;  tableHeight = height;
		contentWidth  = width-4;
		contentHeight = (headerCols > 0) ? height-headerHeight-4 : height-4;
		scrollP.setSize(width, contentHeight+4);
		contentP.setSize(contentWidth, contentHeight);
	}

	public void setBounds(Rectangle r)
	{
		this.setBounds(r.x, r.y, r.width, r.height);
	}

	public void setHeaderWidth(int[] cw)
	{
		if(cw == null || cw.length < 1)
		{	headerCols = 0;
			scrollP.setBounds(0, 0, tableWidth, tableHeight);
			contentWidth  = tableWidth-4;
			contentHeight = tableHeight-4;
			contentP.setSize(contentWidth, contentHeight);
			return;
		}
		else
		{	contentWidth  = tableWidth-4;
			contentHeight = tableHeight-headerHeight-4;
			contentP.setSize(contentWidth, contentHeight);
			scrollP.setBounds(0, headerHeight, tableWidth, tableHeight-headerHeight);
		}

		headerCols = cw.length;
		headerColW = cw;
		headerBtn = new Button[headerCols];
		int sx=3;
		for(int i=0; i < headerCols; i++)
		{	Button tb = new Button();
			tb.setBounds(sx, 2, headerColW[i], headerHeight);
			//tb.setBackground(headerColor);
			//tb.setFont(headerFont);
			headerBtn[i] = tb;
			super.add(tb);
			sx += headerColW[i];
		}
		if(headerCols > cColCount) setColProperties(headerCols, 0, null);
	}

	public void setHeaderText(String[] cl)
	{
		for(int i=0; i < headerCols; i++) headerBtn[i].setLabel(cl[i]);
	}

	//Properties are sequenced as: RowHeight, spLeft, spTop, spRight, spBot
	public void setRowProperties(int pCount, int[] Prop)
	{
		if(pCount > 0) cRowHeight = Prop[0];
		if(pCount > 1) cRowMLeft  = Prop[1];
		if(pCount > 2) cRowMTop   = Prop[2];
		if(pCount > 3) cRowMRight = Prop[3];
		if(pCount > 4) cRowMBot   = Prop[4];
	}

	//Properties are sequenced as: ColWidth, spLeft, spTop, spRight, spBot
	public void setColProperties(int cCount, int pCount, int[][] Prop)
	{
		if(cCount < 1) return;
		cColCount  = cCount;
		cColWidth  = new int[cCount];
		cColMLeft  = new int[cCount];
		cColMTop   = new int[cCount];
		cColMRight = new int[cCount];
		cColMBot   = new int[cCount];
		cColWidthM = new int[cCount];
		int i;
		if(pCount > 0) for(i=0; i < cCount; i++) cColWidth[i]  = Prop[i][0];
		else for(i=0; i < cCount; i++) cColWidth[i] = headerColW[i];
		if(pCount > 1) for(i=0; i < cCount; i++) cColMLeft[i]  = Prop[i][1];
		else for(i=0; i < cCount; i++) cColMLeft[i]  = cRowMLeft;
		if(pCount > 2) for(i=0; i < cCount; i++) cColMTop[i]   = Prop[i][2];
		else for(i=0; i < cCount; i++) cColMTop[i] = cRowMTop;
		if(pCount > 3) for(i=0; i < cCount; i++) cColMRight[i] = Prop[i][3];
		else for(i=0; i < cCount; i++) cColMRight[i] = cRowMRight;
		if(pCount > 4) for(i=0; i < cCount; i++) cColMBot[i]   = Prop[i][4];
		else for(i=0; i < cCount; i++) cColMBot[i] = cRowMBot;

		for(i=0; i < cCount; i++) cColWidthM[i] = cColWidth[i]-cColMLeft[i]-cColMRight[i];
	}

	public void setAnchorColumn(int hc) { hrefCol = hc; }

	public Component add(Component c)
	{
		if(cColCount < 1) return null;

		c.setBackground(tableColorF);

		int i = cursorPos;
		if(i == hrefCol)	//if the column row data acts as an anchor
		{
			c.setForeground(hrefColor);
			c.setFont(hrefFont);
			c.setCursor(AppConfig.Cursor_HAND);
			c.addMouseListener(this);
		}
		else
		{
			//int reqWidth = dataFM.stringWidth(((Label)c).getText());
			//c.setBounds(cursorX+borderT+cColMLeft[i], cursorY+borderT+cColMTop[i],
			//(cColWidthM[i]-borderT < reqWidth) ? cColWidthM[i]-borderT : reqWidth, fontHeight);
			c.setForeground(dataColor);
			c.setFont(dataFont);
		}

		c.setBounds(cursorX+borderT+cColMLeft[i], cursorY+borderT+cColMTop[i],
			cColWidthM[i]-borderT, cRowHeight-borderT-cColMTop[i]-cColMBot[i]);
		contentP.add(c);

		cursorX += cColWidth[cursorPos];
		if(cursorX > contentWidth)
			{ contentWidth = cursorX+1; contentP.setSize(contentWidth, contentHeight); }
		cursorPos++;
		if(cursorPos >= cColCount)
		{	cursorPos=0; cursorX=0;
			cursorY += cRowHeight;
			if(cursorY > contentHeight)
			{ contentHeight = cursorY+1; contentP.setSize(contentWidth, contentHeight); }
		}

		return c;
	}

	public void add(String[] colData)
	{
		int dLen = colData.length;
		for(int i=0; i < dLen; i++)
		{	Label Lb = new Label(colData[i]);
			this.add(Lb);
		}
	}

	public void skipCol(int n)
	{
		for(int i=0; i < n; i++)
		{
		cursorX += cColWidth[cursorPos];
		cursorPos++;
		if(cursorPos >= cColCount)
		{	cursorPos=0; cursorX=0;
			cursorY += cRowHeight;
		}
		}
	}

	public void skipRow(int n)
	{
		cursorPos=0; cursorX=0;
		cursorY += n*cRowHeight;
	}

	public void reset()
	{
		contentWidth  = scrollP.getSize().width-4;
		contentHeight = scrollP.getSize().height-4;
		contentP.setSize(contentWidth, contentHeight);
		contentP.removeAll();

		//Reset Layout
		cRowHeight = 20;
		cRowMLeft  = 0;
		cRowMTop   = 0;
		cRowMRight = 0;
		cRowMBot   = 0;
		if(headerCols > 0) setColProperties(headerCols, 0, null);
		else cColCount  = 0;
		hrefCol = -1;

		cursorPos = 0;
		cursorX   = 0;
		cursorY   = 0;

		this.repaint();
	}

	public void paint(Graphics g)
	{
		g.setColor(headerColor);
		g.fillRect(0, 0, tableWidth, headerHeight);
		g.setColor(Color.gray);
		g.drawLine(0, 0, 0, headerHeight);
		g.drawLine(0, 0, tableWidth, 0);
		g.setColor(Color.black);
		g.drawLine(1, 0, 1, headerHeight);
		g.drawLine(0, 1, tableWidth, 1);
		g.setColor(Color.white);
		g.drawLine(tableWidth-1, 0, tableWidth-1, headerHeight);
	}

	//Event Handler for All Actions
	public void actionPerformed(ActionEvent e)
	{

	}

	//------ Implementation of MouseListener Interface------
	public void mouseClicked(MouseEvent e)
	{
		//System.out.println("MouseClicked");
	}

	public void mouseEntered(MouseEvent e)
	{
		//System.out.println("mouseEntered");
		e.getComponent().setForeground(hrefColorH);
	}

	public void mouseExited(MouseEvent e)
	{
		//System.out.println("mouseExited");
		e.getComponent().setForeground(hrefColor);
	}

	public void mousePressed(MouseEvent e)
	{

	}

	public void mouseReleased(MouseEvent e)
	{

	}
}
