/*
 * @(#)BillingPortalLogin.java	1.5 1.5
 * Copyright (c) iBasis Inc.
*/
//package PortalServlets;

import java.io.*;
import java.util.Enumeration;

import javax.servlet.*;
import javax.servlet.http.*;


public class BillingLoadBalancer extends HttpServlet {

	private static final double MAX_LOADFACTOR = 10;
	private static final int MAX_SERVLETS = 50;
	private static String[] ServletURL   = new String[MAX_SERVLETS];
	private static int[] ServletLoad     = new int[MAX_SERVLETS];
	private static int[] ServletCapacity = new int[MAX_SERVLETS];
	private static boolean[] ServletAvailable = new boolean[MAX_SERVLETS];
	private static int activeServlets = 0;


	public void init()
	{
		//try adding a portal servlet on this server
		//create a temp socket to get IP addr of server because servletHostName is local
		try {
			java.net.Socket ts = new java.net.Socket("www.yahoo.com", 80);
			String IpAddr = ts.getLocalAddress().getHostAddress();
			ts.close();
			//add portal servlet to index 0
			ServletURL[0] = IpAddr+"/servlet/portal";
			ServletAvailable[0] = true;
			ServletLoad[0] = 0;
			ServletCapacity[0] = 400;
			System.out.println("Added Portal Servlet on "+IpAddr+" with Capacity 400 connections");
		}
		catch(Exception e)
		{
			System.out.println("Error opening TCP/IP socket for Portal Servlet: "+e.toString());
			System.out.println("Starting Portal Servlet for localHost");
			ServletURL[0] = "127.0.0.1/servlet/portal";
			ServletAvailable[0] = true;
			ServletLoad[0] = 0;
			ServletCapacity[0] = 400;
		}
	}


	public static int getAvailableServlet()
	{
		int uIndex=0;

		synchronized(ServletURL)
		{
			for(int i =0; i < activeServlets; i++)
			if(ServletAvailable[i])
			{
				double cLoad = ((double)ServletLoad[i])/ServletCapacity[i];
				if( cLoad < MAX_LOADFACTOR ) uIndex = i;
			}
		}

		return uIndex;
	}

	public void reportDeadServlet(String deadServletUrl)
	{
		synchronized(ServletURL)
		{
			for(int i =0; i < activeServlets; i++)
			if(ServletURL[i].equals(deadServletUrl))
			{	ServletAvailable[i] = false;
				break;
			}
		}
	}


	public void doGet (HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		String portalHeader = "<HEAD></HEAD><Frameset rows='*,0' frameborder='no' border='0' framespacing='0'>\n";
		String resMessage;

		//Get the output stream
		ServletOutputStream out = res.getOutputStream();
		res.setContentType("text/html");

		String Message = req.getParameter("Message");
		if(Message == null)
		{	resMessage = "<html><head><title>Load Balancer Servlet</title></head><body>\n<form name=loadbal method=post>"+
							 "\n<h4>URL:&nbsp;</h4><input type=text name=url size=30></input><br>"+
							 "\n<h4>Capacity:&nbsp;</h4><input type=text name=capacity size=30></input><br>"+
							 "\n<input type=hidden name=Message value=AddServlet></input><br>"+
							 "\n<input type=submit name=Add value='Add Servlet'></input>"+
							 "\n<input type=submit name=Remove value='Remove Servlet' onclick=\"document.loadbal.Message.value='RemoveServlet'\"></input></form>";
			out.println(resMessage);
			out.println(getServletList() +"\n</body></html>");
		}
		else if(Message.equalsIgnoreCase("getURL"))
		{
			String fileURL, servURL;
			int servIndex = getAvailableServlet();
			servURL = ServletURL[servIndex];
System.out.println("Servlet URL: "+servURL);
			int urlBase = servURL.indexOf('/');
			if(urlBase > -1) fileURL = servURL.substring(0, urlBase);
			else fileURL = servURL;
//System.out.println(fileURL);
			resMessage = "<FRAME src='http://"+fileURL+"/portal/BillingLogin.html' name='";
			resMessage = resMessage.concat(urlToHex(servURL));
			resMessage = resMessage.concat("' frameborder='0' marginheight='10' marginwidth='10' scrolling='no'>\n");
			resMessage = resMessage.concat("<FRAME src='http://"+fileURL+"/portal/Blank.html' name='TempFrame'");
			resMessage = resMessage.concat(" frameborder='0' marginheight='0' marginwidth='0' scrolling='no'>\n");
			out.println(portalHeader);
//System.out.println(portalHeader);
			out.println(resMessage +"</Frameset></HTML>");
//System.out.println(resMessage +"</Frameset></HTML>");
			ServletLoad[servIndex]++;
		}

		out.close();
	}

	public void doPost (HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		String loadbalHeader = "<HTML><HEAD><TITLE>Load Balancer Servlet</TITLE></HEAD><BODY>";
		String resMessage, RedirectURL;

		//Get the output stream
		ServletOutputStream out = res.getOutputStream();
		res.setContentType("text/html");

		String Message = req.getParameter("Message");
		String sURL= req.getParameter("url");

		if(Message.equalsIgnoreCase("AddServlet"))
		{
			String capString = req.getParameter("capacity");
			if(capString.length() < 1) capString="1";

			int sPos = -1;
			for(int i =0; i < activeServlets; i++)
				if(ServletURL[i].equals(sURL)) { sPos = i; break; }
			if(sPos < 0) { sPos = activeServlets; activeServlets++; }
			ServletURL[sPos] = sURL;
			ServletAvailable[sPos] = true;
			ServletLoad[sPos] = 0;
			try {
			ServletCapacity[sPos] = Integer.parseInt(capString);
			} catch (Exception e) { ServletAvailable[sPos] = false; }

			resMessage = "<p><h3>OK......Servlet successfully added to the list of Active servlets</h3><br>";
			RedirectURL = " The next available Servlet is " + ServletURL[getAvailableServlet()];
			out.println(loadbalHeader+resMessage);
			out.println(getServletList() + "<P>" + RedirectURL+"</BODY></HTML>");
		}
		else if(Message.equalsIgnoreCase("RemoveServlet"))
		{
			int sPos = -1;
			for(int i =0; i < activeServlets; i++)
				if(ServletURL[i].equals(sURL)) { sPos = i; break; }
			if(sPos >= 0) ServletAvailable[sPos] = false;
			resMessage = "<p><h3>OK......Servlet successfully removed from the list of Active servlets</h3><br>";
			out.println(loadbalHeader+resMessage);
			out.println(getServletList() + "</BODY></HTML>");
		}

		out.close();
	}

	private String getServletList()
	{
		String rText="";

		rText = rText.concat("<p><h4>Current List of Billing Servlets:</h4>");
		rText = rText.concat("<Table border='1' cellpadding='3' bgcolor='#DDDDDD'><tr><th>Servlet URL</th><th>Available</th><th>Max Capacity</th><th>Current Load</th></tr>");
		for(int i =0; i < activeServlets; i++)
		{
			if(ServletAvailable[i])
			 rText = rText.concat("<tr><td>"+ServletURL[i]+"</td><td>"+ServletAvailable[i]+"</td><td>"+ServletCapacity[i]+"</td><td>"+ServletLoad[i]+"</td></tr>");
			else
			 rText = rText.concat("<tr bgcolor='red'><td>"+ServletURL[i]+"</td><td>"+ServletAvailable[i]+"</td><td>"+ServletCapacity[i]+"</td><td>"+ServletLoad[i]+"</td></tr>");
		}
		rText = rText.concat("</Table>");
		return rText;
	}


	static synchronized String urlToHex(String sURL)
	{
		StringBuffer tBuf = new StringBuffer();
		char[] uChars = sURL.toCharArray();
		int len = uChars.length;

		for(int i=0; i < len; i++) tBuf.append(Integer.toHexString(uChars[i]));
		return tBuf.toString();
	}

	static String hexToURL(String sHex)
		{
			StringBuffer tBuf = new StringBuffer();
			char[] uChars = sHex.toCharArray();
			int len = uChars.length;

			for(int i=0; i < len; i += 2) tBuf.append((char)(Integer.parseInt(new String(uChars, i, 2), 16)));
			return tBuf.toString();
	}

	public static void main(String[] args)
	{
		String tHex = BillingLoadBalancer.urlToHex("http://205.187.177.164:8080/servlet/portal");
		System.out.println("hex  Value: "+tHex);
		System.out.println("char Value: "+BillingLoadBalancer.hexToURL(tHex));
	}
}


/*
    public void doGet (HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException
    {

	  //Get the session object
	  HttpSession session = req.getSession(true);

	  //Get the output stream
	  ServletOutputStream out = res.getOutputStream();

	  res.setContentType("text/html");

	  out.println("<HEAD><TITLE> SessionServlet Output " +
		      "</TITLE></HEAD><BODY>");
	  out.println("<h1> SessionServlet Output </h1>");
	  out.println("Remote Addr: "+req.getRemoteAddr());
	  out.println("Remote Host: "+req.getRemoteHost());
	  out.println("Server Name: "+req.getServerName());
	  out.println("Server Port: "+req.getServerPort());

	  //Here's the meat
	  Integer ival = (Integer) session.getValue("sessiontest.counter");
	  if (ival==null) ival = new Integer(1);
	  else ival = new Integer(ival.intValue() + 1);
	  session.putValue("sessiontest.counter", ival);

	  out.println("You have hit this page <b>" + ival + "</b> times.<p>");

	  // encodeURLeEncodes the specified URL by including the session ID in it
	  // if cookies are not turned on or not supported by the browser
	  out.println("Click <a href=" + res.encodeURL("/session.html") +
		      ">here</a>");
	  out.println(" to ensure that session tracking is working even if" +
		      " cookies aren't supported.<br>");
	  out.println(" Note that by default URL rewriting is not enabled due" +
		      " to it's expensive overhead.");
	  out.println("<p>");

	  out.println("<h3>Request and Session Data:</h3>");
	  out.println("Session ID in Request: " + req.getRequestedSessionId());
	  out.println("<br>Session ID in Request from Cookie: " +
					req.isRequestedSessionIdFromCookie());
	  out.println("<br>Session ID in Request from URL: " +
					req.isRequestedSessionIdFromURL());
	  out.println("<br>Valid Session ID: " +
					req.isRequestedSessionIdValid());
	  out.println("<h3>Session Data:</h3>");
	  out.println("New Session: " + session.isNew());
	  out.println("<br>Session ID: " + session.getId());
	  out.println("<br>Creation Time: " + session.getCreationTime());
	  out.println("<br>Last Accessed Time: " +
						session.getLastAccessedTime());
	  out.println("<br><a href=\"/examples/simple_servlets\">Up</a>");
	  out.println("</BODY>");
	  out.close();

//int n=1;
//res.sendError(res.SC_SERVICE_UNAVAILABLE);
//if(n > 0) return;
      }
*/
